# -*- coding: utf-8 -*-
"""XMLSchema -- Classes to support processing W3C XML Schema definitions.

This module supports processing DOM model representations of XML schema into a
Python object representation of the schema component model.

The module also provides a top-level class that supports schema processing.
"""

# Get the bindings relevant to schemas.  NB: Other implementations may be used
# in the future.
from pyxb.xmlschema.structures import Schema as schema

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
import sys

# Thanks to agrimstrup for this example

import os.path
xsd='''
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified" version="8 1.87" targetNamespace="URN:test-trac-0057">

  <xsd:element name="ObsProject">
    <xsd:complexType>
      <xsd:sequence>
        <xsd:element name="assignedPriority" type="xsd:int"/>
        <xsd:element name="timeOfCreation" type="xsd:string"/>
      </xsd:sequence>
      <xsd:attribute name="schemaVersion" type="xsd:string" use="required" fixed="8"/>
      <xsd:attribute name="revision" type="xsd:string" default="1.87"/>
      <xsd:attribute name="almatype" type="xsd:string" use="required" fixed="APDM::ObsProject"/>
    </xsd:complexType>
  </xsd:element>


</xsd:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0057 (unittest.TestCase):
    XMLs = '<ns1:ObsProject almatype="APDM::ObsProject" revision="1.74" schemaVersion="8" xmlns:ns1="URN:test-trac-0057"><ns1:timeOfCreation>2009-05-08 21:23:45</ns1:timeOfCreation></ns1:ObsProject>'
    XMLd = XMLs.encode('utf-8')

    def exec_toxmld (self, v):
        return v.toxml("utf-8")

    def tearDown (self):
        pyxb.RequireValidWhenGenerating(True)
        pyxb.RequireValidWhenParsing(True)

    def testDefault (self):
        self.assertTrue(pyxb._GenerationRequiresValid)
        self.assertTrue(pyxb._ParsingRequiresValid)
        self.assertRaises(pyxb.UnrecognizedContentError, CreateFromDocument, self.XMLs)
        doc = pyxb.utils.domutils.StringToDOM(self.XMLs)
        self.assertRaises(pyxb.UnrecognizedContentError, CreateFromDOM, doc)
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(UnrecognizedContentError) as cm:
                CreateFromDocument(self.XMLs)
            # Verify the exception tells us what was being processed
            self.assertTrue(isinstance(cm.exception.instance, ObsProject.typeDefinition()))
            # Verify the exception tells us what was rejected
            time_of_creation_ed = ObsProject.typeDefinition()._UseForTag(Namespace.createExpandedName('timeOfCreation'))
            self.assertTrue(isinstance(cm.exception.value, time_of_creation_ed.elementBinding().typeDefinition()))
            # Verify the exception tells us what would be acceptable
            accept = cm.exception.automaton_configuration.acceptableContent()
            self.assertEqual(1, len(accept))
            assigned_priority_ed = ObsProject.typeDefinition()._UseForTag(Namespace.createExpandedName('assignedPriority'))
            self.assertEqual(accept[0].elementDeclaration(), assigned_priority_ed)

    def testDisable (self):
        pyxb.RequireValidWhenParsing(False)
        instance = CreateFromDocument(self.XMLs)
        self.assertRaises(pyxb.IncompleteElementContentError, self.exec_toxmld, instance)
        if sys.version_info[:2] >= (2, 7):
            with self.assertRaises(IncompleteElementContentError) as cm:
                instance.toxml('utf-8', root_only=True)
            # Verify the exception tells us what was being processed
            self.assertEqual(instance, cm.exception.instance)
            # Verify the exception tells us what would be acceptable
            accept = cm.exception.fac_configuration.acceptableSymbols()
            self.assertEqual(1, len(accept))
            assigned_priority_ed = ObsProject.typeDefinition()._UseForTag(Namespace.createExpandedName('assignedPriority'))
            self.assertEqual(accept[0].elementDeclaration(), assigned_priority_ed)
            # Verify the exception tells us what was left
            time_of_creation_ed = ObsProject.typeDefinition()._UseForTag(Namespace.createExpandedName('timeOfCreation'))
            self.assertEqual(instance.timeOfCreation, cm.exception.symbol_set[time_of_creation_ed][0])

        doc = pyxb.utils.domutils.StringToDOM(self.XMLs)
        instance = CreateFromDOM(doc)
        pyxb.RequireValidWhenGenerating(False)
        xmld = instance.toxml("utf-8", root_only=True)
        self.assertEqual(xmld, self.XMLd)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""PyXB stands for Python U{W3C XML
Schema<http://www.w3.org/XML/Schema>} Bindings, and is pronounced
"pixbee".  It enables translation between XML instance documents and
Python objects following rules specified by an XML Schema document.

This is the top-level entrypoint to the PyXB system.  Importing this
gets you all the L{exceptions<pyxb.exceptions_.PyXBException>}, and
L{pyxb.namespace}.  For more functionality, delve into these
submodules:

 - L{pyxb.xmlschema} Module holding the
   L{structures<pyxb.xmlschema.structures>} that convert XMLSchema
   from a DOM model to a Python class model based on the XMLSchema
   components.  Use this when you need to operate on the component
   model.

 - L{pyxb.binding} Module used to generate the bindings and at runtime
   to support the generated bindings.  Use this if you need to use the
   binding model or content model.

 - L{pyxb.utils} Common utilities used in parsing, generating, and
   executing.  The submodules must be imported separately.

"""

import logging
from pyxb.utils import six

_log = logging.getLogger(__name__)

class cscRoot (object):
    """This little bundle of joy exists because in Python 2.6 it
    became an error to invoke C{object.__init__} with parameters (unless
    you also override C{__new__}, in which case it's only a warning.
    Whatever.).  Since I'm bloody not going to check in every class
    whether C{super(Myclass,self)} refers to C{object} (even if I could
    figure out how to do that, 'cuz the obvious solutions don't work),
    we'll just make this thing the root of all U{cooperative super
    calling<http://www.geocities.com/foetsch/python/new_style_classes.htm#super>}
    hierarchies.  The standard syntax in PyXB for this pattern is::

      def method_csc (self, *args, **kw):
        self_fn = lambda *_args, **_kw: self
        super_fn = getattr(super(ThisClass, self), 'method_csc', self_fn)
        return super_fn(*args, **kw)

    """

    def __init__ (self, *args, **kw):
        # Oh gross.  If this class descends from list (and probably dict), we
        # get here when object is *not* our direct superclass.  In that case,
        # we have to pass the arguments on up, or the strings don't get
        # created right.  Below is the only way I've figured out to detect the
        # situation.
        #
        # Note that we might also get here if you mix-in a class that used
        # object as a parent instead of cscRoot.  Don't do that.  Printing the
        # mro() is a decent way of identifying the problem.
        if issubclass(self.__class__.mro()[-2], ( list, dict )):
            super(cscRoot, self).__init__(*args)

__version__ = '1.2.5'
"""The version of PyXB"""

__url__ = 'http://pyxb.sourceforge.net'
"""The URL for PyXB's homepage"""

__license__ = 'Apache License 2.0'

# Bring in the exception hierarchy
from pyxb.exceptions_ import *

# Bring in namespace stuff
import pyxb.namespace

class BIND (object):
    """Bundle data for automated binding generation.

    Instances of this class capture positional and keyword arguments that are
    used to create binding instances based on context.  For example, if C{w}
    is an instance of a complex type whose C{option} element is declared to be
    an anonymous class with simple content of type integer and an attribute of
    C{units}, a correct assignment to that element could be achieved with::

      w.option = BIND(54, units="m")

    """
    __args = None
    __kw = None

    def __init__ (self, *args, **kw):
        """Cache parameters for subsequent binding creation.
        Invoke just as you would the factory for a binding class."""
        self.__args = args
        self.__kw = kw

    def createInstance (self, factory, **kw):
        """Invoke the given factory method.

        Position arguments to the factory are those cached in this instance.
        Keyword arguments are the ones on the command line, updated from the
        ones in this instance."""
        kw.update(self.__kw)
        return factory(*self.__args, **kw)


XMLStyle_minidom = 0
"""Use xml.dom.minidom for XML processing.  This is the fastest, but does not
provide location information.  It produces DOM instances."""

XMLStyle_saxdom = 1
"""Use pyxb.utils.saxdom for XML processing.  This is the slowest, but both
provides location information and generates a DOM instance."""

XMLStyle_saxer = 2
"""Use pyxb.binding.saxer when converting documents to binding instances.
This style supports location information in the bindings.  It produces binding
instances directly, without going through a DOM stage, so is faster than
XMLStyle_saxdom.  However, since the pyxb.xmlschema.structures classes require
a DOM model, XMLStyle_saxdom will be used for pyxb.utils.domutils.StringToDOM
if this style is selected."""

_XMLStyle = XMLStyle_saxer
"""The current XML processing style."""

_XMLStyleMap = { 'minidom' : XMLStyle_minidom,
                 'saxdom' : XMLStyle_saxdom,
                 'saxer' : XMLStyle_saxer }
_XMLStyleMapReverse = dict([ (_v, _k) for (_k, _v) in six.iteritems(_XMLStyleMap) ])

_XMLStyle_envvar = 'PYXB_XML_STYLE'

def _SetXMLStyle (style=None):
    """Set the interface used to parse XML content.

    This can be invoked within code.  The system default of L{XMLStyle_saxer}
    can also be overridden at runtime by setting the environment variable
    C{PYXB_XML_STYLE} to one of C{minidom}, C{saxdom}, or C{saxer}.

    @param style: One of L{XMLStyle_minidom}, L{XMLStyle_saxdom},
    L{XMLStyle_saxer}.  If not provided, the system default is used.
    """
    global _XMLStyle
    if style is None:
        import os
        style_name = os.environ.get(_XMLStyle_envvar)
        if style_name is None:
            style_name = 'saxer'
        style = _XMLStyleMap.get(style_name)
        if style is None:
            raise PyXBException('Bad value "%s" for %s' % (style_name, _XMLStyle_envvar))
    if _XMLStyleMapReverse.get(style) is None:
        raise PyXBException('Bad value %s for _SetXMLStyle' % (style,))
    _XMLStyle = style

_SetXMLStyle()

# Global flag that we can use to determine whether optimization is active in
# this session.  There may be cases where we can bypass methods that just
# check for things we don't care about in an optimized context
_OptimizationActive = False
try:
    assert False
    _OptimizationActive = True
except:
    pass

_CorruptionDetectionEnabled = not _OptimizationActive
"""If C{True}, blocks attempts to assign to attributes that are reserved for
PyXB methods.

Applies only at compilation time; dynamic changes are ignored.
"""

class ValidationConfig (object):
    """Class holding configuration related to validation.

    L{pyxb.GlobalValidationConfig} is available to influence validation in all
    contexts.  Each binding class has a reference to an instance of this
    class, which can be inspected using
    L{pyxb.binding.basis._TypeBinding_mixin._GetValidationConfig} and changed
    using L{pyxb.binding.basis._TypeBinding_mixin._SetValidationConfig}.  Each
    binding instance has a reference inherited from its class which can be
    inspected using L{pyxb.binding.basis._TypeBinding_mixin._validationConfig}
    and changed using
    L{pyxb.binding.basis._TypeBinding_mixin._setValidationConfig}.

    This allows fine control on a per class and per-instance basis.

    L{forBinding} replaces L{RequireValidWhenParsing}.

    L{forDocument} replaces L{RequireValidWhenGenerating}.

    L{contentInfluencesGeneration}, L{orphanElementInContent}, and
    L{invalidElementInContent} control how
    L{pyxb.binding.basis.complexTypeDefinition.orderedContent} affects
    generated documents.
    """

    __forBinding = True
    def _getForBinding (self):
        """C{True} iff validation should be performed when manipulating a
        binding instance.

        This includes parsing a document or DOM tree, using a binding instance
        class constructor, or assigning to an element or attribute field of a
        binding instance."""
        return self.__forBinding
    def _setForBinding (self, value):
        """Configure whether validation should be performed when manipulating
        a binding instance."""
        if not isinstance(value, bool):
            raise TypeError(value)
        self.__forBinding = value
        return value
    forBinding = property(_getForBinding)

    __forDocument = True
    def _getForDocument (self):
        """C{True} iff validation should be performed when creating a document
        from a binding instance.

        This applies at invocation of
        L{toDOM()<pyxb.binding.basis._TypeBinding_mixin.toDOM>}.
        L{toxml()<pyxb.binding.basis._TypeBinding_mixin.toDOM>} invokes C{toDOM()}."""
        return self.__forDocument
    def _setForDocument (self, value):
        """Configure whether validation should be performed when generating
        a document from a binding instance."""
        if not isinstance(value, bool):
            raise TypeError(value)
        self.__forDocument = value
        return value
    forDocument = property(_getForDocument)

    ALWAYS = -1
    """Always do it."""

    NEVER = 0
    """Never do it."""

    IGNORE_ONCE = 1
    """If an error occurs ignore it and continue with the next one.  (E.g., if
    an element in a content list fails skip it and continue with the next
    element in the list.)"""

    GIVE_UP = 2
    """If an error occurs ignore it and stop using whatever provided the cause
    of the error.  (E.g., if an element in a content list fails stop
    processing the content list and execute as though it was absent)."""

    RAISE_EXCEPTION = 3
    """If an error occurs, raise an exception."""

    MIXED_ONLY = 4
    """Only when content type is mixed."""

    __contentInfluencesGeneration = MIXED_ONLY
    def __getContentInfluencesGeneration (self):
        """Determine whether complex type content influences element order in
        document generation.

        The value is one of L{ALWAYS}, L{NEVER}, L{MIXED_ONLY} (default)."""
        return self.__contentInfluencesGeneration
    def _setContentInfluencesGeneration (self, value):
        """Set the value of L{contentInfluencesGeneration}."""
        if not (value in ( self.ALWAYS, self.NEVER, self.MIXED_ONLY )):
            raise ValueError(value)
        self.__contentInfluencesGeneration = value
    contentInfluencesGeneration = property(__getContentInfluencesGeneration)

    __orphanElementInContent = IGNORE_ONCE
    def __getOrphanElementInContent (self):
        """How to handle unrecognized elements in content lists.

        This is used when consulting a complex type instance content list to
        influence the generation of documents from a binding instance.

        The value is one of L{IGNORE_ONCE} (default), L{GIVE_UP},
        L{RAISE_EXCEPTION}."""
        return self.__orphanElementInContent
    def _setOrphanElementInContent (self, value):
        """Set the value of L{orphanElementInContent}."""
        if not (value in ( self.IGNORE_ONCE, self.GIVE_UP, self.RAISE_EXCEPTION )):
            raise ValueError(value)
        self.__orphanElementInContent = value
    orphanElementInContent = property(__getOrphanElementInContent)

    __invalidElementInContent = IGNORE_ONCE
    def __getInvalidElementInContent (self):
        """How to handle invalid elements in content lists.

        The value is one of L{IGNORE_ONCE} (default), L{GIVE_UP},
        L{RAISE_EXCEPTION}."""
        return self.__invalidElementInContent
    def _setInvalidElementInContent (self, value):
        """Set the value of L{invalidElementInContent}."""
        if not (value in ( self.IGNORE_ONCE, self.GIVE_UP, self.RAISE_EXCEPTION )):
            raise ValueError(value)
        self.__invalidElementInContent = value
    invalidElementInContent = property(__getInvalidElementInContent)

    def copy (self):
        """Make a copy of this instance.

        Use this to get a starting point when you need to customize validation
        on a per-instance/per-class basis."""
        import copy
        return copy.copy(self)

GlobalValidationConfig = ValidationConfig()

_GenerationRequiresValid = True
"""Legacy flag; prefer L{forDocument<ValidationConfig.forDocument>} in L{GlobalValidationConfig}."""

def RequireValidWhenGenerating (value=None):
    """Query or set a flag that controls validation checking in XML generation.

    Normally any attempts to convert a binding instance to a DOM or XML
    representation requires that the binding validate against the content
    model, since only in this way can the content be generated in the correct
    order.  In some cases it may be necessary or useful to generate a document
    from a binding that is incomplete.  If validation is not required, the
    generated documents may not validate even if the content validates,
    because ordering constraints will be ignored.

    @keyword value: If absent or C{None}, no change is made; otherwise, this
    enables (C{True}) or disables (C{False}) the requirement that instances
    validate before being converted to XML.
    @type value: C{bool}

    @return: C{True} iff attempts to generate XML for a binding that does not
    validate should raise an exception.  """
    if value is None:
        return GlobalValidationConfig.forDocument
    global _GenerationRequiresValid
    _GenerationRequiresValid = GlobalValidationConfig._setForDocument(value)
    return value

_ParsingRequiresValid = True
"""Legacy flag; prefer L{forBinding<ValidationConfig.forBinding>} in L{GlobalValidationConfig}."""
def RequireValidWhenParsing (value=None):
    """Query or set a flag that controls validation checking in XML parsing.

    Normally any attempts to convert XML to a binding instance to a binding
    instance requires that the document validate against the content model.
    In some cases it may be necessary or useful to process a document that is
    incomplete.  If validation is not required, the generated documents may
    not validate even if the content validates, because ordering constraints
    will be ignored.

    @keyword value: If absent or C{None}, no change is made; otherwise, this
    enables (C{True}) or disables (C{False}) the requirement that documents
    validate when being converted to bindings.
    @type value: C{bool}

    @return: C{True} iff attempts to generate bindings for a document that
    does not validate should raise an exception."""
    if value is None:
        return GlobalValidationConfig.forBinding
    global _ParsingRequiresValid
    _ParsingRequiresValid = GlobalValidationConfig._setForBinding(value)
    return _ParsingRequiresValid

_PreserveInputTimeZone = False
def PreserveInputTimeZone (value=None):
    """Control whether time values are converted to UTC during input.

    The U{specification <http://www.w3.org/TR/xmlschema-2/#dateTime>} makes
    clear that timezoned times are in UTC and that times in other timezones
    are to be translated to UTC when converted from literal to value form.
    Provide an option to bypass this step, so the input timezone is preserved.

    @note: Naive processing of unnormalized times--i.e., ignoring the
    C{tzinfo} field--may result in errors."""
    global _PreserveInputTimeZone
    if value is None:
        return _PreserveInputTimeZone
    if not isinstance(value, bool):
        raise TypeError(value)
    _PreserveInputTimeZone = value
    return _PreserveInputTimeZone

_OutputEncoding = 'utf-8'
"""Default unicode encoding to use when creating output.

Material being written to an XML parser is not output."""

_InputEncoding = 'utf-8'
"""Default unicode encoding to assume when decoding input.

Material being written to an XML parser is treated as input."""

def NonElementContent (instance):
    """Return an iterator producing the non-element content of the provided instance.

    The catenated text of the non-element content of an instance can
    be obtained with::

       text = six.u('').join(pyxb.NonElementContent(instance))

    @param instance: An instance of L{pyxb.binding.basis.complexTypeDefinition}.

    @return: an iterator producing text values
    """
    import pyxb.binding.basis
    return pyxb.binding.basis.NonElementContent.ContentIterator(instance.orderedContent())

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""This module contains support for generating bindings from an XML stream
using a SAX parser."""

import logging
import xml.dom
import pyxb.namespace
import pyxb.utils.saxutils
import pyxb.utils.saxdom
import pyxb.utils.utility
from pyxb.binding import basis
from pyxb.namespace.builtin import XMLSchema_instance as XSI

_log = logging.getLogger(__name__)

class _SAXElementState (pyxb.utils.saxutils.SAXElementState):
    """State required to generate bindings for a specific element.

    If the document being parsed includes references to unrecognized elements,
    a DOM instance of the element and its content is created and treated as a
    wildcard element.
    """

    # An expanded name corresponding to xsi:nil
    __XSINilTuple = XSI.nil.uriTuple()

    # The binding instance being created for this element.  When the
    # element type has simple content, the binding instance cannot be
    # created until the end of the element has been reached and the
    # content of the element has been processed accumulated for use in
    # the instance constructor.  When the element type has complex
    # content, the binding instance must be created at the start of
    # the element, so contained elements can be properly stored.
    __bindingInstance = None

    # The schema binding for the element being constructed.
    __elementBinding = None

    def setElementBinding (self, element_binding):
        """Record the binding to be used for this element.

        Generally ignored, except at the top level this is the only way to
        associate a binding instance created from an xsi:type description with
        a specific element."""
        self.__elementBinding = element_binding

    # The nearest enclosing complex type definition
    def enclosingCTD (self):
        """The nearest enclosing complex type definition, as used for
        resolving local element/attribute names.

        @return: An instance of L{basis.complexTypeDefinition}, or C{None} if
        the element is top-level
        """
        return self.__enclosingCTD
    __enclosingCTD = None

    # The factory that is called to create a binding instance for this
    # element; None if the binding instance was created at the start
    # of the element.
    __delayedConstructor = None

    # An xml.sax.xmlreader.Attributes instance providing the
    # attributes for the element.
    __attributes = None

    # An xml.dom.Node corresponding to the (sub-)document
    __domDocument = None

    __domDepth = None

    def __init__ (self, **kw):
        super(_SAXElementState, self).__init__(**kw)
        self.__bindingInstance = None
        parent_state = self.parentState()
        if isinstance(parent_state, _SAXElementState):
            self.__enclosingCTD = parent_state.enclosingCTD()
            self.__domDocument = parent_state.__domDocument
            if self.__domDocument is not None:
                self.__domDepth = parent_state.__domDepth + 1

    def setEnclosingCTD (self, enclosing_ctd):
        """Set the enclosing complex type definition for this element.

        @param enclosing_ctd: The scope for a local element.
        @type enclosing_ctd: L{basis.complexTypeDefinition}
        @return: C{self}
        """
        self.__enclosingCTD = enclosing_ctd

    # Create the binding instance for this element.
    def __constructElement (self, new_object_factory, attrs, content=None):
        kw = { '_from_xml' : True,
               '_location' : self.location() }

        # Note whether the node is marked nil
        if self.__XSINilTuple in attrs:
            kw['_nil'] = pyxb.binding.datatypes.boolean(attrs.getValue(self.__XSINilTuple))

        if content is None:
            content = []
        self.__bindingInstance = new_object_factory(*content, **kw)
        if isinstance(self.__bindingInstance, pyxb.utils.utility.Locatable_mixin):
            self.__bindingInstance._setLocation(self.location())

        # Record the namespace context so users of the binding can
        # interpret QNames within the attributes and content.
        self.__bindingInstance._setNamespaceContext(self.__namespaceContext)

        # Set instance attributes
        # NB: attrs implements the SAX AttributesNS interface, meaning
        # that names are pairs of (namespaceURI, localName), just like we
        # want them to be.
        for attr_name in self.__attributes.getNames():
            attr_en = pyxb.namespace.ExpandedName(attr_name)
            # Ignore xmlns and xsi attributes; we've already handled those
            if attr_en.namespace() in ( pyxb.namespace.XMLNamespaces, XSI ):
                continue
            # The binding instance may be a simple type that does not support
            # attributes; the following raises an exception in that case.
            self.__bindingInstance._setAttribute(attr_en, attrs.getValue(attr_name))

        return self.__bindingInstance

    def inDOMMode (self):
        return self.__domDocument is not None

    def enterDOMMode (self, attrs):
        """Actions upon first encountering an element for which we cannot create a binding.

        Invoking this transitions the parser into DOM mode, creating a new DOM
        document that will represent this element including its content."""
        assert not self.__domDocument
        self.__domDocument = pyxb.utils.saxdom.Document(namespace_context=self.namespaceContext())
        self.__domDepth = 0
        return self.startDOMElement(attrs)

    def startDOMElement (self, attrs):
        """Actions upon entering an element that is part of a DOM subtree."""
        self.__domDepth += 1
        self.__attributes = pyxb.utils.saxdom.NamedNodeMap()
        ns_ctx = self.namespaceContext()
        for name in attrs.getNames():
            attr_en = pyxb.namespace.ExpandedName(name)
            self.__attributes._addItem(pyxb.utils.saxdom.Attr(expanded_name=attr_en, namespace_context=ns_ctx, value=attrs.getValue(name), location=self.location()))

    def endDOMElement (self):
        """Actions upon leaving an element that is part of a DOM subtree."""
        ns_ctx = self.namespaceContext()
        element = pyxb.utils.saxdom.Element(namespace_context=ns_ctx, expanded_name=self.expandedName(), attributes=self.__attributes, location=self.location())
        for info in self.content():
            if isinstance(info.item, xml.dom.Node):
                element.appendChild(info.item)
            else:
                element.appendChild(pyxb.utils.saxdom.Text(info.item, namespace_context=ns_ctx))
        self.__domDepth -= 1
        if 0 == self.__domDepth:
            self.__domDocument.appendChild(element)
            #pyxb.utils.saxdom._DumpDOM(self.__domDocument)
            self.__domDepth = None
            self.__domDocument = None
        parent_state = self.parentState()
        parent_state.addElementContent(self.location(), element, None)
        return element

    def startBindingElement (self, type_class, new_object_factory, element_decl, attrs):
        """Actions upon entering an element that will produce a binding instance.

        The element use is recorded.  If the type is a subclass of
        L{basis.simpleTypeDefinition}, a delayed constructor is recorded so
        the binding instance can be created upon completion of the element;
        otherwise, a binding instance is created and stored.  The attributes
        are used to initialize the binding instance (now, or upon element
        end).

        @param type_class: The Python type of the binding instance
        @type type_class: subclass of L{basis._TypeBinding_mixin}
        @param new_object_factory: A callable object that creates an instance of the C{type_class}
        @param element_decl: The element use with which the binding instance is associated.  Will be C{None} for top-level elements
        @type element_decl: L{basis.element}
        @param attrs: The XML attributes associated with the element
        @type attrs: C{xml.sax.xmlreader.Attributes}
        @return: The generated binding instance, or C{None} if creation is delayed
        """
        self.__delayedConstructor = None
        self.__elementDecl = element_decl
        self.__attributes = attrs
        if type_class._IsSimpleTypeContent():
            self.__delayedConstructor = new_object_factory
        else:
            try:
                pyxb.namespace.NamespaceContext.PushContext(self.namespaceContext())
                self.__constructElement(new_object_factory, attrs)
            finally:
                pyxb.namespace.NamespaceContext.PopContext()
        return self.__bindingInstance

    def endBindingElement (self):
        """Perform any end-of-element processing.

        For simple type instances, this creates the binding instance.
        @return: The generated binding instance
        """
        if self.__delayedConstructor is not None:
            args = []
            for info in self.content():
                if info.maybe_element or (info.element_decl is not None):
                    raise pyxb.NonElementValidationError(info.item, info.location)
                args.append(info.item)
            try:
                pyxb.namespace.NamespaceContext.PushContext(self.namespaceContext())
                self.__constructElement(self.__delayedConstructor, self.__attributes, args)
            except pyxb.ValidationError as e:
                if e.location is None:
                    e.location = self.location()
                raise
            finally:
                pyxb.namespace.NamespaceContext.PopContext()
        else:
            for info in self.content():
                self.__bindingInstance.append(info.item,
                                              _element_decl=info.element_decl,
                                              _maybe_element=info.maybe_element,
                                              _location=info.location)
        parent_state = self.parentState()
        if parent_state is not None:
            parent_state.addElementContent(self.location(), self.__bindingInstance, self.__elementDecl)
        # As CreateFromDOM does, validate the resulting element
        if self.__bindingInstance._element() is None:
            self.__bindingInstance._setElement(self.__elementBinding)
        return self.__bindingInstance._postDOMValidate()

class PyXBSAXHandler (pyxb.utils.saxutils.BaseSAXHandler):
    """A SAX handler class which generates a binding instance for a document
    through a streaming parser.

    An example of using this to parse the document held in the (unicode) text
    value C{xmlt} is::

      import pyxb.binding.saxer
      import io

      saxer = pyxb.binding.saxer.make_parser()
      handler = saxer.getContentHandler()
      saxer.parse(io.StringIO(xmlt))
      instance = handler.rootObject()

    """

    # Whether invocation of handler methods should be traced
    __trace = False

    # An expanded name corresponding to xsi:type
    __XSITypeTuple = XSI.type.uriTuple()

    __domHandler = None
    __domDepth = None

    def rootObject (self):
        """Return the binding object corresponding to the top-most
        element in the document

        @return: An instance of L{basis._TypeBinding_mixin} (most usually a
        L{basis.complexTypeDefinition}.

        @raise pyxb.[UnrecognizedDOMRootNodeError: No binding could be found to
        match the top-level element in the document."""
        if not isinstance(self.__rootObject, basis._TypeBinding_mixin):
            # Happens if the top-level element got processed as a DOM instance.
            assert isinstance(self.__rootObject, xml.dom.Node)
            raise pyxb.UnrecognizedDOMRootNodeError(self.__rootObject)
        return self.__rootObject._postDOMValidate()
    __rootObject = None

    def reset (self):
        """Reset the state of the handler in preparation for processing a new
        document.

        @return: C{self}
        """
        super(PyXBSAXHandler, self).reset()
        self.__rootObject = None
        return self

    def __init__ (self, **kw):
        """Create a parser instance for converting XML to bindings.

        @keyword element_state_constructor: Overridden with the value
        L{_SAXElementState} before invoking the L{superclass
        constructor<pyxb.utils.saxutils.BaseSAXHandler.__init__>}.
        """

        kw.setdefault('element_state_constructor', _SAXElementState)
        super(PyXBSAXHandler, self).__init__(**kw)
        self.reset()

    def startElementNS (self, name, qname, attrs):
        (this_state, parent_state, ns_ctx, name_en) = super(PyXBSAXHandler, self).startElementNS(name, qname, attrs)

        # Delegate processing if in DOM mode
        if this_state.inDOMMode():
            return this_state.startDOMElement(attrs)

        # Resolve the element within the appropriate context.  Note
        # that global elements have no use, only the binding.
        if parent_state.enclosingCTD() is not None:
            (element_binding, element_decl) = parent_state.enclosingCTD()._ElementBindingDeclForName(name_en)
        else:
            element_decl = None
            element_binding = name_en.elementBinding()
        this_state.setElementBinding(element_binding)

        # Non-root elements should have an element use, from which we can
        # extract the binding if we couldn't find one elsewhere.  (Keep any
        # current binding, since it may be a member of a substitution group.)
        if (element_decl is not None) and (element_binding is None):
            assert self.__rootObject is not None
            element_binding = element_decl.elementBinding()
            assert element_binding is not None

        # Start knowing nothing
        type_class = None
        if element_binding is not None:
            element_binding = element_binding.elementForName(name)
            type_class = element_binding.typeDefinition()

        # Process an xsi:type attribute, if present
        if self.__XSITypeTuple in attrs:
            (did_replace, type_class) = XSI._InterpretTypeAttribute(attrs.getValue(self.__XSITypeTuple), ns_ctx, self.fallbackNamespace(), type_class)
            if did_replace:
                element_binding = None

        if type_class is None:
            # Bother.  We don't know what this thing is.  But that's not an
            # error, if the schema accepts wildcards.  For consistency with
            # the DOM-based interface, we need to build a DOM node.
            return this_state.enterDOMMode(attrs)

        if element_binding is not None:
            # Invoke binding __call__ method not Factory, so can check for
            # abstract elements.
            new_object_factory = element_binding
        else:
            new_object_factory = type_class.Factory

        # Update the enclosing complex type definition for this
        # element state.
        assert type_class is not None
        if issubclass(type_class, pyxb.binding.basis.complexTypeDefinition):
            this_state.setEnclosingCTD(type_class)
        else:
            this_state.setEnclosingCTD(parent_state.enclosingCTD())

        # Process the element start.  This may or may not return a
        # binding object.
        binding_object = this_state.startBindingElement(type_class, new_object_factory, element_decl, attrs)

        # If the top-level element has complex content, this sets the
        # root object.  If it has simple content, see endElementNS.
        if self.__rootObject is None:
            self.__rootObject = binding_object

    def endElementNS (self, name, qname):
        this_state = super(PyXBSAXHandler, self).endElementNS(name, qname)
        if this_state.inDOMMode():
            # Delegate processing if in DOM mode.  Note that completing this
            # element may take us out of DOM mode.  In any case, the returned
            # binding object is a DOM element instance.
            binding_object = this_state.endDOMElement()
        else:
            # Process the element end.  This will return a binding object,
            # either the one created at the start or the one created at
            # the end.
            binding_object = this_state.endBindingElement()
        assert binding_object is not None

        # If we don't have a root object, save it.  No, there is not a
        # problem doing this on the close of the element.  If the
        # top-level element has complex content, the object was
        # created on start, and the root object has been assigned.  If
        # it has simple content, then there are no internal elements
        # that could slip in and set this before we get to it here.
        #
        # Unless we're still in DOM mode, in which case this isn't really the
        # root object.  Then the real root object will be the one that caused
        # us to enter DOM mode.
        if (self.__rootObject is None) and not this_state.inDOMMode():
            self.__rootObject = binding_object

def make_parser (*args, **kw):
    """Extend L{pyxb.utils.saxutils.make_parser} to change the default
    C{content_handler_constructor} to be L{PyXBSAXHandler}.
    """
    kw.setdefault('content_handler_constructor', PyXBSAXHandler)
    return pyxb.utils.saxutils.make_parser(*args, **kw)

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.utils.utility
import pyxb.binding.datatypes as xsd

import unittest

class TestTrac0206 (unittest.TestCase):
    Time = "2013-08-30T11:56:45+04:00" # = 2013-08-30T07:56:45Z

    def setUp (self):
        self.__pitz = pyxb.PreserveInputTimeZone()
        self.__ltz = xsd.dateTime._LocalTimeZone
        xsd.dateTime._LocalTimeZone = pyxb.utils.utility.UTCOffsetTimeZone(120)

    def tearDown (self):
        pyxb.PreserveInputTimeZone(self.__pitz)
        xsd.dateTime._LocalTimeZone = self.__ltz

    def testBasic (self):
        self.assertFalse(pyxb.PreserveInputTimeZone())
        dt = xsd.dateTime(self.Time)
        self.assertEqual('2013-08-30 07:56:45+00:00', str(dt))
        self.assertEqual('2013-08-30 09:56:45+02:00', str(dt.aslocal()))

    def testPreserve (self):
        pyxb.PreserveInputTimeZone(True)
        self.assertTrue(pyxb.PreserveInputTimeZone())
        dt = xsd.dateTime(self.Time)
        self.assertEqual('2013-08-30 11:56:45+04:00', str(dt))
        self.assertEqual('2013-08-30 09:56:45+02:00', str(dt.aslocal()))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xs:complexType name="empty"/>
    <xs:element name="elt" type="empty"/>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestIssue0031 (unittest.TestCase):
    def testAttribute (self):
        with self.assertRaises(pyxb.UnrecognizedAttributeError) as cm:
            CreateFromDocument('<elt atr="foo"/>')
        self.assertEqual(str(cm.exception), 'Attempt to reference unrecognized attribute atr in type {}'.format(empty))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node
import pyxb.namespace
import sys
import imp

import os.path

te_generator = pyxb.binding.generate.Generator(allow_absent_module=True, generate_to_files=False)
te_generator.setSchemaRoot(os.path.realpath('%s/../schemas' % (os.path.dirname(__file__),)))
te_generator.addSchemaLocation('test-external.xsd')

# Create a module into which we'll stick the shared types bindings.
# Put it into the sys modules so the import directive in subsequent
# code is resolved.
st = imp.new_module('st')
sys.modules['st'] = st

# Now get the code for the shared types bindings, and evaluate it
# within the new module.

st_generator = pyxb.binding.generate.Generator(allow_absent_module=True, generate_to_files=False)
st_generator.setSchemaRoot(os.path.realpath('%s/../schemas' % (os.path.dirname(__file__),)))
st_generator.addSchemaLocation('shared-types.xsd')

st_modules = st_generator.bindingModules()
assert 1 == len(st_modules)
code = st_modules.pop().moduleContents()
open('st.py', 'w').write(code)
rv = compile(code, 'shared-types', 'exec')
six.exec_(code, st.__dict__)

# Set the path by which we expect to reference the module
stns = pyxb.namespace.NamespaceForURI('URN:shared-types', create_if_missing=True)
module_record = stns.lookupModuleRecordByUID(st_generator.generationUID())
assert module_record is not None, 'Unable to find %s in ns %s' % (st_generator.generationUID(), stns)
module_record.setModulePath('st')

# Now get and build a module that refers to that module.

modules = te_generator.bindingModules()
assert 1 == len(modules)
m = modules.pop()
assert m.namespace() != stns
code = m.moduleContents()
open('te.py', 'w').write(code)
rv = compile(code, 'test-external', 'exec')
eval(rv)

from pyxb.exceptions_ import *

from pyxb.utils import domutils
def ToDOM (instance):
    return instance.toDOM().documentElement

import unittest

class TestExternal (unittest.TestCase):
    def setUp (self):
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level

    def tearDown (self):
        self.__basis_log.level = self.__basis_loglevel

    def testSharedTypes (self):
        self.assertEqual(word.typeDefinition()._ElementMap['from'].elementBinding().typeDefinition(), st.english)
        self.assertEqual(word.typeDefinition()._ElementMap['to'].elementBinding().typeDefinition(), st.welsh)
        one = st.english('one')
        self.assertRaises(SimpleTypeValueError, st.english, 'five')
        # Element constructor without content is error
        self.assertRaises(SimpleTypeValueError, english)
        self.assertEqual('one', english('one'))
        # Element constructor with out-of-range content is error
        self.assertRaises(SimpleTypeValueError, english, 'five')

        xmlt = six.u('<ns1:english xmlns:ns1="URN:test-external">one</ns1:english>')
        xmld = xmlt.encode('utf-8')
        instance = st.CreateFromDocument(xmlt)
        self.assertEqual('one', instance)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testWords (self):
        xmlt = six.u('<ns1:word xmlns:ns1="URN:test-external"><from>one</from><to>un</to></ns1:word>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual('one', instance.from_)
        self.assertEqual('un', instance.to)
        self.assertEqual(ToDOM(instance).toxml("utf-8"), xmld)

    def testBadWords (self):
        xmlt = six.u('<ns1:word xmlns:ns1="URN:test-external"><from>five</from><to>pump</to></ns1:word>')
        xmld = xmlt.encode('utf-8')
        self.assertRaises(SimpleTypeValueError, CreateFromDocument, xmlt)
        self.assertRaises(SimpleTypeValueError, CreateFromDocument, xmld)

    def testComplexShared (self):
        xmlt = six.u('<ns1:lwords language="english" newlanguage="welsh" xmlns:ns1="URN:test-external">un</ns1:lwords>')
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance._element(), lwords)
        self.assertTrue(isinstance(instance.value(), st.welsh))
        self.assertEqual('english', instance.language)
        self.assertEqual('welsh', instance.newlanguage)

    def testCrossedRestriction (self):
        # Content model elements that are consistent with parent
        # should share its fields; those that change something should
        # override it.
        self.assertEqual(st.extendedName._ElementMap['title'], restExtName._ElementMap['title'])
        self.assertEqual(st.extendedName._ElementMap['forename'], restExtName._ElementMap['forename'])
        self.assertEqual(st.extendedName._ElementMap['surname'], restExtName._ElementMap['surname'])
        self.assertEqual(st.extendedName._ElementMap['generation'], restExtName._ElementMap['generation'])

        xmlt = six.u('<personName><surname>Smith</surname></personName>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = st.personName.Factory(_dom_node=dom.documentElement)
        xmlt = six.u('<personName><surname>Smith</surname><generation>Jr.</generation></personName>')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        self.__basis_log.setLevel(logging.ERROR)
        self.assertRaises(UnrecognizedContentError, st.personName.Factory, _dom_node=dom.documentElement)
        self.__basis_log.level = self.__basis_loglevel
        xmlt = xmlt.replace('personName', 'extendedName')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = st.extendedName.Factory(_dom_node=dom.documentElement)
        xmlt = xmlt.replace('extendedName', 'restExtName')
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = restExtName.Factory(_dom_node=dom.documentElement)

    def testUnionExtension (self):
        e = morewords('one')
        self.assertTrue(isinstance(e, st.english))
        self.assertTrue(uMorewords._IsValidValue(e))
        self.assertEqual(e, st.english.one)
        self.assertEqual(e, uMorewords.one)
        w = morewords('un')
        self.assertTrue(isinstance(w, st.welsh))
        self.assertTrue(uMorewords._IsValidValue(w))
        self.assertEqual(w, st.welsh.un)
        self.assertEqual(w, uMorewords.un)
        n = morewords('ichi')
        self.assertTrue(uMorewords._IsValidValue(n))
        self.assertEqual(n, uMorewords.ichi)

if __name__ == '__main__':
    unittest.main()

import os
try:
    os.unlink('st.py')
    os.unlink('te.py')
except:
    pass



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''
<schema
    xmlns="http://www.w3.org/2001/XMLSchema"
    targetNamespace="urn:issue0032"
    xmlns:tns="urn:issue0032"
    elementFormDefault="qualified">

    <element name="customer">
        <complexType>
            <sequence>
                <element name="address" type="tns:address"/>
            </sequence>
        </complexType>
    </element>

    <complexType name="address">
        <sequence>
            <element name="street" type="string"/>
        </sequence>
    </complexType>

    <complexType name="canadianAddress">
        <complexContent>
            <extension base="tns:address">
                <sequence>
                    <element name="postalCode" type="string"/>
                </sequence>
            </extension>
        </complexContent>
    </complexType>

</schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestIssue0032 (unittest.TestCase):
    xmlt = '''<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<customer xmlns="urn:issue0032">
    <address xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="canadianAddress">
        <street>1 A Street</street>
        <postalCode>Ontario</postalCode>
    </address>
</customer>'''

    def testRoundTrip (self):
        i = CreateFromDocument(self.xmlt)
        self.assertTrue(isinstance(i.address, canadianAddress))
        xmlt = i.toxml('utf-8')
        i2 = CreateFromDocument(xmlt)
        self.assertTrue(isinstance(i2.address, canadianAddress))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
import pyxb.binding.saxer
import io

from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/xsi-nil.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

def setFull (instance, value):
    instance.full = value
    return instance

class TestXSIType (unittest.TestCase):

    def testFull (self):
        xmlt = six.u('<full xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">content</full>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())
        self.assertRaises(pyxb.NoNillableSupportError, instance._setIsNil)

        xmlt = six.u('<full xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="false">content</full>')
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())

        xmlt = six.u('<full xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true">content</full>')
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())

    def testXFull (self):
        xmlt = six.u('<xfull xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">content</xfull>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())
        self.assertRaises(pyxb.NoNillableSupportError, instance._setIsNil)

    def testOptional (self):
        xmlt = six.u('<optional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">content</optional>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())
        instance._setIsNil()
        self.assertTrue(instance._isNil())

        saxer = pyxb.binding.saxer.make_parser(fallback_namespace=Namespace)
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        instance = handler.rootObject()
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())

    def testOptionalNilFalse (self):
        xmlt = six.u('<optional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="false">content</optional>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, 'content')
        self.assertFalse(instance._isNil())
        instance._setIsNil()
        self.assertTrue(instance._isNil())

    def testOptionalNilEETag (self):
        xmlt = six.u('<optional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, '')
        self.assertTrue(instance._isNil())

        saxer = pyxb.binding.saxer.make_parser(fallback_namespace=Namespace)
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        instance = handler.rootObject()
        self.assertEqual(instance, '')
        self.assertTrue(instance._isNil())

    def testOptionalNilSETag (self):
        xmlt = six.u('<optional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"></optional>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, '')
        self.assertTrue(instance._isNil())

        saxer = pyxb.binding.saxer.make_parser(fallback_namespace=Namespace)
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        instance = handler.rootObject()
        self.assertEqual(instance, '')
        self.assertTrue(instance._isNil())

    def testOptionalNilSCETag (self):
        xmlt = six.u('<optional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"><!-- comment --></optional>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance, '')
        self.assertTrue(instance._isNil())

        saxer = pyxb.binding.saxer.make_parser(fallback_namespace=Namespace)
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        instance = handler.rootObject()
        self.assertEqual(instance, '')
        self.assertTrue(instance._isNil())

    def testNilOptionalSpaceContent (self):
        xmlt = six.u('<optional xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"> </optional>')
        self.assertRaises(pyxb.ContentInNilInstanceError, CreateFromDocument, xmlt)

    def testNilComplexSpaceContent (self):
        xmlt = six.u('<complex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"> </complex>')
        self.assertRaises(pyxb.ContentInNilInstanceError, CreateFromDocument, xmlt)

    def testComplexInternal (self):
        xmlt = six.u('<complex><full>full content</full><optional>optional content</optional></complex>')
        xmld = xmlt.encode('utf-8')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance.full, 'full content')
        self.assertEqual(instance.optional, 'optional content')
        self.assertFalse(instance.optional._isNil())
        self.assertEqual(instance.toDOM().documentElement.toxml("utf-8"), xmld)
        instance.validateBinding()

        saxer = pyxb.binding.saxer.make_parser(fallback_namespace=Namespace)
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        instance = handler.rootObject()
        self.assertEqual(instance.full, 'full content')
        self.assertEqual(instance.optional, 'optional content')
        self.assertFalse(instance.optional._isNil())

        xmlt = six.u('<complex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><full>full content</full><optional xsi:nil="true"></optional></complex>')
        xmld = xmlt.encode('utf-8')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(instance.full, 'full content')
        self.assertEqual(instance.optional, '')
        self.assertTrue(instance.optional._isNil())
        self.assertEqual(instance.toDOM().documentElement.toxml("utf-8"), xmld)
        instance.validateBinding()

        saxer = pyxb.binding.saxer.make_parser(fallback_namespace=Namespace)
        handler = saxer.getContentHandler()
        saxer.parse(io.StringIO(xmlt))
        instance = handler.rootObject()
        self.assertEqual(instance.full, 'full content')
        self.assertEqual(instance.optional, '')
        self.assertTrue(instance.optional._isNil())

        xmlt = six.u('<complex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        xmld = xmlt.encode('utf-8')
        instance._setIsNil()
        self.assertEqual(instance.toDOM().documentElement.toxml("utf-8"), xmld)
        instance.validateBinding()

    def testComplex (self):
        canonicalt = six.u('<complex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        canonicald = canonicalt.encode('utf-8')
        for xmlt in ( canonicalt,
                      six.u('<complex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"></complex>'),
                      six.u('<complex xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"><!-- comment --></complex>')) :
            doc = pyxb.utils.domutils.StringToDOM(xmlt)
            instance = CreateFromDOM(doc.documentElement)
            self.assertTrue(instance._isNil())
            self.assertEqual(instance.toDOM().documentElement.toxml("utf-8"), canonicald)
            instance.validateBinding()

    def testConstructorNil (self):
        o = optional()
        self.assertFalse(o._isNil())
        o = optional(_nil=True)
        self.assertTrue(o._isNil())
        c = complex()
        self.assertFalse(c._isNil())
        c = complex(_nil=True)
        self.assertTrue(c._isNil())
        s = simple()
        self.assertFalse(s._isNil())
        s = simple(_nil=True)
        self.assertTrue(s._isNil())

    def testNilSetAndAppend (self):
        s = simple(_nil=True)
        self.assertRaises(pyxb.ContentInNilInstanceError, s.append, 'one')
        c = complex(_nil=True)
        self.assertRaises(pyxb.ContentInNilInstanceError, c.append, 'one')
        full = c._UseForTag('full')
        self.assertRaises(pyxb.ContentInNilInstanceError, setFull, c, 'yes')
        self.assertRaises(pyxb.ContentInNilInstanceError, full.append, c, 'yes')
        self.assertRaises(pyxb.ContentInNilInstanceError, full.setOrAppend, c, 'yes')
        multi = c._UseForTag('multi')
        self.assertRaises(pyxb.ContentInNilInstanceError, multi.append, c, 'one')
        self.assertRaises(pyxb.ContentInNilInstanceError, multi.setOrAppend, c, 'one')
        # Nothing we can do about assignments that bypass the validation hooks.
        # Customizing list would probably destroy performance.
        c.multi.append('one')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest
import B
from pyxb import BIND

class Trac43_2 (unittest.TestCase):

    def testCode (self):
        myobj = B.b1(BIND('a2m-value', 'legal'), 'legal')
        self.assertEqual('a2m-value', myobj.a2elt.a2member)
        self.assertEqual(B.bst.legal, myobj.a2elt.a2b)

        myobj.a2elt = BIND('anotherValue', 'legal')
        self.assertEqual('anotherValue', myobj.a2elt.a2member)
        self.assertEqual(B.bst.legal, myobj.a2elt.a2b)

unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-facets.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestFacets (unittest.TestCase):
    def testQuantity (self):
        xml = '<quantity xmlns="URN:test-facets">35</quantity>'
        instance = CreateFromDOM(pyxb.utils.domutils.StringToDOM(xml).documentElement)
        self.assertEqual(35, instance)
        self.assertTrue(isinstance(instance, quantity.typeDefinition()))
        self.assertRaises(Exception, quantity, -52)
        self.assertRaises(Exception, quantity, 100)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xst = '''<?xml version="1.0"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="topLevel">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="item" type="xs:int" maxOccurs="unbounded"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0218 (unittest.TestCase):
    def testBasic (self):
        instance = topLevel()
        self.assertTrue(instance.item is not None)
        self.assertFalse(instance.item is None)
        self.assertTrue(instance.item != None)
        self.assertTrue(None != instance.item)
        self.assertFalse(instance.item)
        instance.item.extend([1,2,3,4])
        self.assertTrue(instance.item is not None)
        self.assertFalse(instance.item is None)
        self.assertTrue(instance.item != None)
        self.assertTrue(None != instance.item)
        self.assertTrue(instance.item)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="xdateTime" nillable="true" type="xs:dateTime"/>
  <xs:element name="xdate" nillable="true" type="xs:date"/>
  <xs:element name="xtime" nillable="true" type="xs:time"/>
  <xs:element name="xboolean" nillable="true" type="xs:boolean"/>
  <xs:element name="xduration" nillable="true" type="xs:duration"/>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0200 (unittest.TestCase):

    def testFull_dateTime (self):
        instance = CreateFromDocument('<xdateTime>2006-05-04T18:13:51.0Z</xdateTime>')
        self.assertFalse(instance._isNil())

    def testNil_dateTime (self):
        instance = CreateFromDocument('<xdateTime xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        self.assertTrue(instance._isNil())

    def testFull_date (self):
        instance = CreateFromDocument('<xdate>2006-05-04</xdate>')
        self.assertFalse(instance._isNil())

    def testNil_date (self):
        instance = CreateFromDocument('<xdate xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        self.assertTrue(instance._isNil())

    def testFull_time (self):
        instance = CreateFromDocument('<xtime>18:13:51.0Z</xtime>')
        self.assertFalse(instance._isNil())

    def testNil_time (self):
        instance = CreateFromDocument('<xtime xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        self.assertTrue(instance._isNil())

    def testFull_boolean (self):
        instance = CreateFromDocument('<xboolean>true</xboolean>')
        self.assertFalse(instance._isNil())

    def testNil_boolean (self):
        instance = CreateFromDocument('<xboolean xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        self.assertTrue(instance._isNil())

    def testFull_duration (self):
        instance = CreateFromDocument('<xduration>P3D</xduration>')
        self.assertFalse(instance._isNil())

    def testNil_duration (self):
        instance = CreateFromDocument('<xduration xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
        self.assertTrue(instance._isNil())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/absentns.xsd'))

xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema targetNamespace="URN:trac0046" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:tns="URN:trac0046">
  <xs:include schemaLocation="%s"/>
  <xs:element name="anotherGlobalComplex" type="tns:structure"/>
</xs:schema>''' % (schema_path,)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
rv = compile(code, 'test', 'exec')
eval(rv)

import unittest

class TestTrac_0046 (unittest.TestCase):
    def testParsing (self):
        g1 = globalComplex('simple')
        g2 = anotherGlobalComplex('simple2', g1)
        self.assertEqual(g2.globalSimple, 'simple2')
        self.assertEqual(g2.innerComplex.globalSimple, 'simple')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:tns="urn:trac-0071" targetNamespace="urn:trac-0071">
  <xs:element name="MetadataDocument" type="tns:MetadataType"/>
  <xs:complexType name="MetadataType">
    <xs:sequence maxOccurs="1" minOccurs="1">
      <xs:element name="template" type="xs:string"/>
      <xs:element name="timespan" maxOccurs="unbounded" minOccurs="0">
        <xs:complexType>
          <xs:sequence>
            <xs:element name="field" maxOccurs="unbounded" minOccurs="0">
              <xs:complexType>
                <xs:sequence>
                  <xs:element name="name" type="xs:string"/>
                  <xs:element name="value" minOccurs="0" maxOccurs="unbounded">
                    <xs:complexType>
                      <xs:simpleContent>
                        <xs:extension base="xs:string">
                          <xs:attribute name="lang" type="xs:language"/>
                          <xs:attribute name="user" type="xs:string"/>
                          <xs:attribute name="timestamp" type="xs:dateTime"/>
                        </xs:extension>
                      </xs:simpleContent>
                    </xs:complexType>
                  </xs:element>
                </xs:sequence>
                <xs:attribute name="track" type="xs:string"/>
              </xs:complexType>
            </xs:element>
          </xs:sequence>
          <xs:attribute name="start" type="xs:string"/>
          <xs:attribute name="end" type="xs:string"/>
        </xs:complexType>
      </xs:element>
    </xs:sequence>
  </xs:complexType>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest
import collections

# Pretend whoever created the schema was helpful and had normalized it
metadatadoc_type = MetadataDocument.typeDefinition()
timespan_element = metadatadoc_type._ElementMap['timespan'].elementBinding()
timespan_type = timespan_element.typeDefinition()
field_element = timespan_type._ElementMap['field'].elementBinding()
field_type = field_element.typeDefinition()
value_element = field_type._ElementMap['value'].elementBinding()
value_type = value_element.typeDefinition()

v_bind = pyxb.BIND('foo', lang='ENG')

class TestTrac_0071 (unittest.TestCase):
    def testFieldConstructor (self):
        field = field_type('title', pyxb.BIND('foo', lang='ENG'), _element=field_element)
        self.assertTrue(isinstance(field.value_, collections.MutableSequence))
        self.assertEqual(1, len(field.value_))
        self.assertTrue(isinstance(field.value_[0], value_type))
        field.validateBinding()
        xmlt = six.u('<field><name>title</name><value lang="ENG">foo</value></field>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(field.toxml("utf-8", root_only=True), xmld)

    def testFieldElementAppend (self):
        newdoc = MetadataDocument()
        newdoc.template = 'anewtemplate'

        field = field_type(name='title', _element=field_element)
        field.value_.append(pyxb.BIND('foo', lang='ENG'))
        self.assertTrue(isinstance(field.value_, collections.MutableSequence))
        self.assertEqual(1, len(field.value_))
        self.assertTrue(isinstance(field.value_[0], value_type))
        field.validateBinding()
        self.assertTrue(isinstance(field.value_[0], value_type))
        xmlt = six.u('<field><name>title</name><value lang="ENG">foo</value></field>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(field.toxml("utf-8", root_only=True), xmld)

    MetaExpectedt = '<ns1:MetadataDocument xmlns:ns1="urn:trac-0071"><template>anewtemplate</template><timespan end="+INF" start="-INF"><field><name>title</name><value lang="ENG">foo</value></field></timespan></ns1:MetadataDocument>'
    MetaExpectedd = MetaExpectedt.encode('utf-8')

    def testMetaConstructor (self):
        newdoc = MetadataDocument()
        newdoc.template = 'anewtemplate'

        newdoc.timespan.append(pyxb.BIND( # Single timespan
                pyxb.BIND( # First field instance
                    'title',
                    pyxb.BIND('foo', lang='ENG')
                    ),
                start='-INF', end='+INF'))
        timespan = newdoc.timespan[0]
        self.assertTrue(isinstance(timespan, timespan_type))
        newdoc.validateBinding()
        timespan = newdoc.timespan[0]
        self.assertTrue(isinstance(timespan, timespan_type))
        self.assertEqual(self.MetaExpectedd, newdoc.toxml("utf-8", root_only=True))
        newdoc.timespan[:] = []

    def testMetaBadFieldName (self):
        newdoc = MetadataDocument()
        newdoc.template = 'anewtemplate'

        v_bind = pyxb.BIND('foo', lang='ENG')

        # This binding is wrong: the field name is "value_" not "value"
        f_bind = pyxb.BIND(name='title', value=v_bind)
        ts_bind = pyxb.BIND(f_bind, start='-INF', end='+INF')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.UnrecognizedContentError, newdoc.timespan.append, ts_bind)
            return
        with self.assertRaises(pyxb.UnrecognizedContentError) as cm:
            newdoc.timespan.append(ts_bind)
        self.assertEqual(f_bind, cm.exception.value)

    def testMetaBadPlurality (self):
        newdoc = MetadataDocument()
        newdoc.template = 'anewtemplate'

        # This binding is wrong: the field "value_" is plural and the
        # value for the keyword must be iterable.
        f_bind = pyxb.BIND(name='title', value_=v_bind)
        ts_bind = pyxb.BIND(f_bind, start='-INF', end='+INF')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.UnrecognizedContentError, newdoc.timespan.append, ts_bind)
            return
        with self.assertRaises(pyxb.UnrecognizedContentError) as cm:
            newdoc.timespan.append(ts_bind)
        self.assertEqual(f_bind, cm.exception.value)

    def testMetaGoodBind (self):
        newdoc = MetadataDocument()
        newdoc.template = 'anewtemplate'

        # This one should be OK
        bind = pyxb.BIND( # First field instance
                    name='title',
                    value_=[pyxb.BIND('foo', lang='ENG')]
                    )
        newdoc.timespan.append(pyxb.BIND(bind, start='-INF', end='+INF'))
        timespan = newdoc.timespan[0]
        self.assertTrue(isinstance(timespan, timespan_type))
        newdoc.validateBinding()
        self.assertEqual(self.MetaExpectedd, newdoc.toxml("utf-8", root_only=True))

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import pyxb
import po4
import address
import pyxb.binding.datatypes as xs
import datetime

po = po4.purchaseOrder(orderDate=xs.date(1999, 10, 20))
po.shipTo = address.USAddress('Alice Smith', '123 Maple Street', 'Anytown', 'AK', 12341)
po.billTo = address.USAddress('Robert Smith', '8 Oak Avenue', 'Anytown', 'AK', 12341)
po.items = pyxb.BIND(pyxb.BIND('Lapis necklace', 1, 99.95, partNum='833-AA'),
                     pyxb.BIND('Plastic necklace', 4, 3.95, partNum='833-AB'))

po.shipTo.country = po.billTo.country = po.shipTo.country

lapis = po.items.item[0]
lapis.shipDate = po.orderDate + datetime.timedelta(days=46)
lapis.comment = 'Want this for the holidays!'
po.items.item[1].shipDate = po.items.item[0].shipDate + datetime.timedelta(days=19)

import pyxb.utils.domutils
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(address.Namespace, 'addr')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(po4.Namespace, 'po')

print(po.toxml("utf-8"))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class Test_ENTITY (unittest.TestCase):
    def testValid (self):
        valid = [ 'schema', '_Underscore', '_With.Dot', 'With-Hyphen' ]
        for f in valid:
            self.assertEqual(f, xsd.ENTITY(f))

    def testInvalid (self):
        invalid = [ '.DotFirst', 'With Spaces', 'With:Colon',
                    'With?Illegal', '??LeadingIllegal', 'TrailingIllegal??',
                    '  LeadingSpace', 'TrailingSpace   ']
        for f in invalid:
            self.assertRaises(SimpleTypeValueError, xsd.ENTITY, f)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="var" type="xs:string"/>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestPyXBCDATA (unittest.TestCase):
    def testParse(self):
        instance = CreateFromDocument("<var>text</var>")
        self.assertEqual(instance, 'text')
        instance = CreateFromDocument("<var><![CDATA[text]]></var>")
        self.assertEqual(instance, 'text')
        instance = CreateFromDocument("<var>&gt; text &lt;</var>")
        self.assertEqual(instance, '> text <')
        instance = CreateFromDocument("<var><![CDATA[> text <]]></var>")
        self.assertEqual(instance, '> text <')

    def testGenerate(self):
        instance = var('text')
        self.assertEqual(instance, 'text')
        instance = var('>text<')
        self.assertEqual(instance, '>text<')
        xmlt = six.u('<var>&gt;text&lt;</var>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb

print("\n".join([ str(_ns) for _ns in pyxb.namespace.AvailableNamespaces() ]))
ns = pyxb.namespace.NamespaceForURI('URN:shared-types', True)
ns.validateComponentModel()
ns = pyxb.namespace.NamespaceForURI('URN:test-external', True)
ns.validateComponentModel()

import bindings.st as st
import bindings.te as te

import unittest

class ExternalTest (unittest.TestCase):
    def testUnionExtension (self):
        e = te.morewords('one')
        self.assertTrue(isinstance(e, st.english))
        self.assertTrue(te.uMorewords._IsValidValue(e))
        self.assertEqual(e, st.english.one)
        self.assertEqual(e, te.uMorewords.one)
        w = te.morewords('un')
        self.assertTrue(isinstance(w, st.welsh))
        self.assertTrue(te.uMorewords._IsValidValue(w))
        self.assertEqual(w, st.welsh.un)
        self.assertEqual(w, te.uMorewords.un)
        n = te.morewords('ichi')
        self.assertTrue(te.uMorewords._IsValidValue(n))
        self.assertEqual(n, te.uMorewords.ichi)

    def testValidation (self):
        self.assertEqual(st.Namespace.uri(), 'URN:shared-types')
        self.assertTrue(st.Namespace.validateComponentModel())
        self.assertEqual(te.Namespace.uri(), 'URN:test-external')
        self.assertTrue(te.Namespace.validateComponentModel())

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.binding.content
import pyxb.utils.domutils
import xml.dom.minidom

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:simpleType name="tInner">
  <xs:restriction base="xs:string"/>
</xs:simpleType>
<xs:complexType name="tTop">
  <xs:sequence>
    <xs:element name="inner" type="tInner"/>
  </xs:sequence>
</xs:complexType>
<xs:element name="top" type="tTop"/>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0075 (unittest.TestCase):
    def setUp (self):
        # Prepare to hide warnings about failure to convert DOM node to binding
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level

    def tearDown (self):
        self.__basis_log.level = self.__basis_loglevel

    def testGood (self):
        xmls = '<top><inner>content</inner></top>'
        instance = CreateFromDocument(xmls)
        self.assertEqual('content', instance.inner)

    def testUnrecognizedDOMRootNodeError (self):
        xmls = '<t0p><inner>content</inner></t0p>'
        try:
            instance = CreateFromDocument(xmls)
            self.fail("Succeeded in creating from document with bad top level element")
        except UnrecognizedDOMRootNodeError as e:
            self.assertEqual('t0p', e.node_name)

    def testNotAnElementError (self):
        elt = tTop._UseForTag('inner')
        self.assertTrue(isinstance(elt, pyxb.binding.content.ElementDeclaration))
        self.assertRaises(KeyError, tTop._UseForTag, 'notInner')

    def testUnrecognizedContentError (self):
        # Hide the warnings that tInner could not be converted to binding
        self.__basis_log.setLevel(logging.ERROR)
        tag = Namespace.createExpandedName('tInner')
        xmlt = '<top><tInner>content</tInner></top>'
        try:
            instance = CreateFromDocument(xmlt)
            self.fail("Succeeded in creating from document with bad inner element")
        except UnrecognizedContentError as e:
            pass

        dom = xml.dom.minidom.parseString(xmlt)
        try:
            instance = CreateFromDOM(dom)
            self.fail("Succeeded in creating from document with bad inner element")
        except UnrecognizedContentError as e:
            pass

    '''
    NOT YET FINISHED

    def testUnrecognizedContentError (self):
        self.fail("Unimplemented test")

    def testMissingContentError (self):
        self.fail("Unimplemented test")

    def testUnrecognizedAttributeError (self):
        self.fail("Unimplemented test")
    '''

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class Test_language (unittest.TestCase):
    def testValid (self):
        valid = [ 'english', 'welsh', 'french-english', 'one-two-three-four' ]
        for f in valid:
            self.assertEqual(f, xsd.language(f))

    def testInvalid (self):
        invalid = [ 'toomanychars', 'illegal_chars', 'numb3rs' ]
        for f in invalid:
            self.assertRaises(SimpleTypeValueError, xsd.language, f)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""This module contains support for a DOM tree representation from an XML
document using a SAX parser.

This functionality exists because we need a DOM interface to generate the
binding classses, but the Python C{xml.dom.minidom} package does not support
location information.  The SAX interface does, so we have a SAX content
handler which converts the SAX events into a DOM tree.

This is not a general-purpose DOM capability; only a small subset of the DOM
interface is supported, and only for storing the XML information, not for
converting it back into document format.
"""

from __future__ import print_function
import logging
import io
import xml.dom
import pyxb.utils.saxutils
from pyxb.utils import six
import pyxb.namespace

_log = logging.getLogger(__name__)

def _DumpDOM (n, depth=0):
    """Utility function to print a DOM tree."""

    pfx = ' ' * depth
    if (xml.dom.Node.ELEMENT_NODE == n.nodeType):
        print('%sElement[%d] %s %s with %d children' % (pfx, n._indexInParent(), n, pyxb.namespace.ExpandedName(n.name), len(n.childNodes)))
        ins = pyxb.namespace.NamespaceContext.GetNodeContext(n).inScopeNamespaces()
        print('%s%s' % (pfx, ' ; '.join([ '%s=%s' % (_k, _v.uri()) for (_k, _v) in ins.items()])))
        for (k, v) in six.iteritems(n.attributes):
            print('%s %s=%s' % (pfx, pyxb.namespace.ExpandedName(k), v))
        for cn in n.childNodes:
            _DumpDOM(cn, depth+1)
    elif (xml.dom.Node.TEXT_NODE == n.nodeType):
        #print '%sText "%s"' % (pfx, n.value)
        pass
    elif (xml.dom.Node.DOCUMENT_NODE == n.nodeType):
        print('Document node')
        _DumpDOM(n.firstChild, depth)
    else:
        print('UNRECOGNIZED %s' % (n.nodeType,))

class _DOMSAXHandler (pyxb.utils.saxutils.BaseSAXHandler):
    """SAX handler class that transforms events into a DOM tree."""

    def document (self):
        """The document that is the root of the generated tree."""
        return self.__document
    __document = None

    def startDocument (self):
        super(_DOMSAXHandler, self).startDocument()
        self.__document = Document(namespace_context=self.namespaceContext())

    def endDocument (self):
        content = self.elementState().content()
        if 0 < len(content):
            assert content[0].maybe_element
            self.__document.appendChild(content[0].item)
            #_DumpDOM(content)

    def startElementNS (self, name, qname, attrs):
        (this_state, parent_state, ns_ctx, name_en) = super(_DOMSAXHandler, self).startElementNS(name, qname, attrs)
        this_state.__attributes = NamedNodeMap()
        for name in attrs.getNames():
            attr_en = pyxb.namespace.ExpandedName(name)
            value = attrs.getValue(name)
            this_state.__attributes._addItem(Attr(expanded_name=attr_en, namespace_context=ns_ctx, value=value, location=this_state.location()))

    def endElementNS (self, name, qname):
        this_state = super(_DOMSAXHandler, self).endElementNS(name, qname)
        ns_ctx = this_state.namespaceContext()
        element = Element(namespace_context=ns_ctx, expanded_name=this_state.expandedName(), attributes=this_state.__attributes, location=this_state.location())
        for info in this_state.content():
            if isinstance(info.item, Node):
                element.appendChild(info.item)
            else:
                element.appendChild(Text(info.item, namespace_context=ns_ctx))
        parent_state = this_state.parentState()
        parent_state.addElementContent(this_state.location(), element, None)

def parse (stream, **kw):
    """Parse a stream containing an XML document and return the DOM tree
    representing its contents.

    Keywords not described here are passed to L{pyxb.utils.saxutils.make_parser}.

    @param stream: An object presenting the standard file C{read} interface
    from which the document can be read.  The content should be data, not text.

    @keyword content_handler_constructor: Input is overridden to assign this a
    value of L{_DOMSAXHandler}.

    @rtype: C{xml.dom.Document}
    """

    kw['content_handler_constructor'] = _DOMSAXHandler
    saxer = pyxb.utils.saxutils.make_parser(**kw)
    handler = saxer.getContentHandler()
    saxer.parse(stream)
    return handler.document()

def parseString (xml_text, **kw):
    """Parse a string holding an XML document and return the corresponding DOM
    tree.

    @param xml_text: the XML content to be parsed, in a text representation."""
    # SAX parser operates on data, not text.
    xmld = xml_text
    if isinstance(xmld, six.text_type):
        xmld = xmld.encode(pyxb._InputEncoding)
    return parse(io.BytesIO(xmld), **kw)

class Node (xml.dom.Node, pyxb.utils.utility.Locatable_mixin):
    """Base for the minimal DOM interface required by PyXB."""
    def __init__ (self, node_type, **kw):
        location = kw.pop('location', None)
        if location is not None:
            pyxb.utils.utility.Locatable_mixin.__init__(self, location=location)
        self.__nodeType = node_type
        self.__parentNode = None
        self.__indexInParent = None
        self.__childNodes = []
        self.__namespaceContext = kw['namespace_context']
        self.__value = kw.get('value')
        self.__attributes = kw.get('attributes')
        en = kw.get('expanded_name')
        if en is not None:
            self.__expandedName = en
            ns = en.namespace()
            if (ns is not None) and not (ns.prefix() is None):
                assert not ns.isAbsentNamespace()
                self.__prefix = ns.prefix()
            if (ns is not None):
                self.__namespaceURI = ns.uri()
            self.__localName = en.localName()
            if self.__prefix:
                self.__tagName = '%s:%s' % (self.__prefix, self.__localName)
            else:
                self.__tagName = self.__localName
        self.__namespaceContext.setNodeContext(self)

    location = property(lambda _s: _s._location())

    __expandedName = None
    _expandedName = property(lambda _s: _s.__expandedName)
    __prefix = ''
    prefix = property(lambda _s: _s.__prefix)
    __tagName = ''
    tagName = property(lambda _s: _s.__tagName)
    __namespaceURI = None
    namespaceURI = property(lambda _s: _s.__namespaceURI)
    __localName = None
    localName = property(lambda _s: _s.__localName)
    __value = None
    value = property(lambda _s: _s.__value)

    def _indexInParent (self): return self.__indexInParent

    def __childIfPresent (self, index):
        if index < len(self.__childNodes):
            return self.__childNodes[index]
        return None

    def appendChild (self, new_child):
        new_child._setParentNode(self, len(self.__childNodes))
        self.__childNodes.append(new_child)

    def _setParentNode (self, parent_node, index_in_parent):
        self.__parentNode = parent_node
        self.__indexInParent = index_in_parent

    def _setAttributes (self, attributes):
        assert self.__attributes is None
        self.__attributes = attributes
    __attributes = None

    nodeType = property(lambda _s: _s.__nodeType)
    parentNode = property(lambda _s: _s.__parentNode)
    firstChild = property(lambda _s: _s.__childIfPresent(0))
    childNodes = property(lambda _s: _s.__childNodes)
    attributes = property(lambda _s: _s.__attributes)

    nextSibling = property(lambda _s: _s.parentNode.__childIfPresent(_s.__indexInParent+1))

    def hasAttributeNS (self, ns_uri, local_name):
        return self.getAttributeNodeNS(ns_uri, local_name) is not None

    def getAttributeNodeNS (self, ns_uri, local_name):
        return self.__attributes._getAttr(pyxb.namespace.ExpandedName(ns_uri, local_name))

    def getAttributeNS (self, ns_uri, local_name):
        rv = self.getAttributeNodeNS(ns_uri, local_name)
        if rv is None:
            return ''
        return rv.value

class Document (Node):
    """Add the documentElement interface."""
    def __init__ (self, **kw):
        super(Document, self).__init__(node_type=xml.dom.Node.DOCUMENT_NODE, **kw)

    documentElement = Node.firstChild

class Attr (Node):
    """Add the nodeName and nodeValue interface."""
    def __init__ (self, **kw):
        super(Attr, self).__init__(node_type=xml.dom.Node.ATTRIBUTE_NODE, **kw)
    name = Node.tagName
    nodeName = Node.tagName
    nodeValue = Node.value

class NamedNodeMap (object):
    """Implement that portion of NamedNodeMap required to satisfy PyXB's
    needs."""
    __members = None
    __memberMap = None

    def __init__ (self):
        super(NamedNodeMap, self).__init__()
        self.__members = []
        self.__memberMap = {}

    length = property(lambda _s: len(_s.__members))
    def item (self, index):
        return self.__members[index]

    def _addItem (self, attr):
        assert pyxb.namespace.NamespaceContext.GetNodeContext(attr) is not None
        self.__members.append(attr)
        en = attr._expandedName
        if en is not None:
            self.__memberMap[en] = attr

    def _getAttr (self, name):
        rv = self.__memberMap.get(name)
        if rv is not None:
            return rv
        return None

class Element (Node):
    def __init__ (self, **kw):
        super(Element, self).__init__(node_type=xml.dom.Node.ELEMENT_NODE, **kw)
        assert self.attributes is not None
    tagName = Node.localName
    nodeName = Node.localName

class _CharacterData (Node):
    """Abstract base for anything holding text data."""
    data = Node.value

class Text (_CharacterData):
    def __init__ (self, text, **kw):
        super(Text, self).__init__(value=text, node_type=xml.dom.Node.TEXT_NODE, **kw)

class Comment (_CharacterData):
    def __init__ (self, text, **kw):
        super(Comment, self).__init__(value=text, node_type=xml.dom.Node.COMMENT_NODE, **kw)

if '__main__' == __name__:
    import sys
    xml_file = 'examples/tmsxtvd/tmsdatadirect_sample.xml'
    if 1 < len(sys.argv):
        xml_file = sys.argv[1]

    doc = parse(open(xml_file))

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:simpleType name="tla">
  <xs:annotation><xs:documentation>Simple type to represent a three-letter acronym</xs:documentation></xs:annotation>
  <xs:restriction base="xs:string">
    <xs:length value="3"/>
  </xs:restriction>
</xs:simpleType>
<xs:simpleType name="Atla">
  <xs:annotation><xs:documentation>A three-letter acronym that starts with A</xs:documentation></xs:annotation>
  <xs:restriction base="tla">
    <xs:pattern value="A.."/>
  </xs:restriction>
</xs:simpleType>
<xs:simpleType name="tlaZ">
  <xs:annotation><xs:documentation>A three-letter acronym that ends with Z</xs:documentation></xs:annotation>
  <xs:restriction base="tla">
    <xs:pattern value="..Z"/>
  </xs:restriction>
</xs:simpleType>
<xs:simpleType name="combAtlaZ">
  <xs:annotation><xs:documentation>A three-letter acronym that either starts with A or ends with Z</xs:documentation></xs:annotation>
  <xs:restriction base="tla">
    <xs:pattern value="A.."/>
    <xs:pattern value="..Z"/>
  </xs:restriction>
</xs:simpleType>
<xs:simpleType name="dervAtlaZ">
  <xs:annotation><xs:documentation>A three-letter acronym that starts with A and ends with Z</xs:documentation></xs:annotation>
  <xs:restriction base="Atla">
    <xs:pattern value="..Z"/>
  </xs:restriction>
</xs:simpleType>

</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0061 (unittest.TestCase):
    def testDocString (self):
        self.assertEqual("Simple type to represent a three-letter acronym", tla._Documentation.strip())
        self.assertEqual("Simple type to represent a three-letter acronym", tla.__doc__.strip())

    def testTLA (self):
        self.assertEqual("tla", tla('tla'))
        self.assertRaises(pyxb.SimpleTypeValueError, tla, 'four')
        self.assertRaises(pyxb.SimpleTypeValueError, tla, '1')

    def testAtla (self):
        self.assertRaises(pyxb.SimpleTypeValueError, Atla, 'four')
        self.assertRaises(pyxb.SimpleTypeValueError, Atla, '1')
        self.assertEqual("A23", Atla('A23'))
        self.assertEqual("A2Z", Atla('A2Z'))
        self.assertRaises(pyxb.SimpleTypeValueError, Atla, 'B12')

    def testtlaZ (self):
        self.assertRaises(pyxb.SimpleTypeValueError, tlaZ, 'four')
        self.assertRaises(pyxb.SimpleTypeValueError, tlaZ, '1')
        self.assertEqual("12Z", tlaZ('12Z'))
        self.assertEqual("A2Z", tlaZ('A2Z'))
        self.assertRaises(pyxb.SimpleTypeValueError, tlaZ, '12X')

    def testcombAtlaZ (self):
        self.assertRaises(pyxb.SimpleTypeValueError, combAtlaZ, 'four')
        self.assertRaises(pyxb.SimpleTypeValueError, combAtlaZ, '1')
        self.assertEqual("A2Z", combAtlaZ('A2Z'))
        self.assertEqual("A23", combAtlaZ('A23'))
        self.assertEqual("12Z", combAtlaZ('12Z'))
        self.assertRaises(pyxb.SimpleTypeValueError, combAtlaZ, '12X')
        self.assertRaises(pyxb.SimpleTypeValueError, combAtlaZ, 'X23')

    def testdervAtlaZ (self):
        self.assertRaises(pyxb.SimpleTypeValueError, dervAtlaZ, 'four')
        self.assertRaises(pyxb.SimpleTypeValueError, dervAtlaZ, '1')
        self.assertEqual("A2Z", dervAtlaZ('A2Z'))
        self.assertRaises(pyxb.SimpleTypeValueError, dervAtlaZ, 'A23')
        self.assertRaises(pyxb.SimpleTypeValueError, dervAtlaZ, '12Z')
        self.assertRaises(pyxb.SimpleTypeValueError, dervAtlaZ, '12X')
        self.assertRaises(pyxb.SimpleTypeValueError, dervAtlaZ, 'X23')

if __name__ == '__main__':
    unittest.main()



from pyxb.bundles.wssplat.raw.soapbind11 import *
import pyxb.bundles.wssplat.raw.soapbind11 as raw_soapbind11
from pyxb.bundles.wssplat.wsdl11 import _WSDL_binding_mixin, _WSDL_port_mixin, _WSDL_operation_mixin

class tBinding (raw_soapbind11.tBinding, _WSDL_binding_mixin):
    pass
raw_soapbind11.tBinding._SetSupersedingClass(tBinding)

class tAddress (raw_soapbind11.tAddress, _WSDL_port_mixin):
    pass
raw_soapbind11.tAddress._SetSupersedingClass(tAddress)

class tOperation (raw_soapbind11.tOperation, _WSDL_operation_mixin):
    def __getLocationInformation (self):
        rvl = []
        if self.soapAction is not None:
            rvl.append('action=%s' % (self.soapAction,))
        if self.style is not None:
            rvl.append('style=%s' % (self.style,))
        return ','.join(rvl)
    locationInformation = property(__getLocationInformation)

raw_soapbind11.tOperation._SetSupersedingClass(tOperation)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import unittest
import pyxb.binding.datatypes as xsd
import datetime

class Test_date (unittest.TestCase):

    def verifyTime (self, dt, with_usec=True, with_adj=(0,0), with_tzinfo=True):
        self.assertEqual(2002, dt.year)
        self.assertEqual(10, dt.month)
        self.assertEqual(27, dt.day)
        self.assertEqual(with_tzinfo, dt.tzinfo is not None)

    def testBad (self):
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.date, '2002-10-27T')

    def testFromText (self):
        self.verifyTime(xsd.date('  2002-10-27', _from_xml=True), with_usec=False, with_tzinfo=False)
        self.verifyTime(xsd.date('2002-10-27  ', _from_xml=True), with_usec=False, with_tzinfo=False)
        self.verifyTime(xsd.date('2002-10-27'), with_usec=False, with_tzinfo=False)

    def testYear (self):
        # This test can't succeed because Python doesn't support negative years.
        self.assertRaises(pyxb.SimpleTypeValueError, xsd.date, '-0024-01-01')

    def testArguments (self):
        self.assertRaises(TypeError, xsd.date)
        self.assertRaises(TypeError, xsd.date, 2002)
        self.assertRaises(TypeError, xsd.date, 2002, 10)
        self.verifyTime(xsd.date(2002, 10, 27), with_tzinfo=False)

    def testXsdLiteral (self):
        dt = xsd.date('2002-10-27')
        self.assertEqual('2002-10-27', dt.xsdLiteral())

    def testTimezoned (self):
        dt = xsd.date('2002-10-10Z')
        self.assertEqual('2002-10-10T00:00:00+00:00', dt.isoformat())
        self.assertEqual('2002-10-10Z', dt.xsdLiteral())

        dt = xsd.date('2002-10-10+13:00')
        self.assertEqual('2002-10-10T00:00:00+13:00', dt.isoformat())
        self.assertEqual('2002-10-09-11:00', dt.xsdLiteral())
        dt = xsd.date('2002-10-09-11:00')
        self.assertEqual('2002-10-09T00:00:00-11:00', dt.isoformat())
        self.assertEqual('2002-10-09-11:00', dt.xsdLiteral())

        dt = xsd.date('2002-10-10+14:00')
        self.assertEqual('2002-10-10T00:00:00+14:00', dt.isoformat())
        self.assertEqual('2002-10-09-10:00', dt.xsdLiteral())
        dt = xsd.date('2002-10-09-10:00')
        self.assertEqual('2002-10-09T00:00:00-10:00', dt.isoformat())
        self.assertEqual('2002-10-09-10:00', dt.xsdLiteral())

        dt = xsd.date('2002-10-10-14:00')
        self.assertEqual('2002-10-10T00:00:00-14:00', dt.isoformat())
        self.assertEqual('2002-10-11+10:00', dt.xsdLiteral())
        dt = xsd.date('2002-10-11+10:00')
        self.assertEqual('2002-10-11T00:00:00+10:00', dt.isoformat())
        self.assertEqual('2002-10-11+10:00', dt.xsdLiteral())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
import gc

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="instance">
    <xs:complexType>
      <xs:all>
         <xs:element name="inner" maxOccurs="unbounded">
           <xs:complexType>
             <xs:all>
               <xs:element name="text" type="xs:string"/>
               <xs:element name="number" type="xs:integer"/>
             </xs:all>
           </xs:complexType>
         </xs:element>
      </xs:all>
    </xs:complexType>
  </xs:element>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

import os

class TestBug_200908271556(unittest.TestCase):
    # Somebody cares, so make this OS-dependent.
    __statm = None
    if sys.platform.startswith('linux'):
        __statm = open('/proc/%d/statm' % (os.getpid(),))

    def __getMem (self):
        if self.__statm is not None:
            self.__statm.seek(0)
            return int(self.__statm.read().split()[0])
        raise NotImplementedError()

    def testMemory (self):
        # Only proceed if the OS provides a way to get the process memory
        try:
            self.__getMem()
        except NotImplementedError:
            _log.warning('%s: test not supported on platform', __file__)
            return
        xmls = '<instance><inner><text>text</text><number>45</number></inner></instance>'
        base_at = 10
        check_at = 20
        growth_limit = 1.10
        iter = 0
        gc.collect()
        while True:
            iter += 1
            if base_at == iter:
                gc.collect()
                base_mem = self.__getMem()
            elif check_at == iter:
                gc.collect()
                check_mem = self.__getMem()
                growth = check_mem - base_mem
                self.assertTrue(0 == growth, 'growth %s' % (growth,))
                break
            instance = CreateFromDocument(xmls)
            xmls = instance.toxml("utf-8", root_only=True)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class Test_NMTOKEN (unittest.TestCase):
    def testValid (self):
        valid = [ 'schema', '_Underscore', '_With.Dot', 'With-Hyphen',
                  'With:Colon', '.DotFirst' ]
        for f in valid:
            self.assertEqual(f, xsd.NMTOKEN(f))

    def testInvalid (self):
        invalid = [ 'With Spaces',
                    'With?Illegal', '??LeadingIllegal', 'TrailingIllegal??',
                    '  LeadingSpace', 'TrailingSpace   ']
        for f in invalid:
            try:
                xsd.NMTOKEN(f)
                print('Unexpected success with %s' % (f,))
            except:
                pass
            self.assertRaises(SimpleTypeValueError, xsd.NMTOKEN, f)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.utils.utility
import pyxb.binding.datatypes as xsd
from pyxb.utils.six.moves import cPickle as pickle

import unittest

class TestTrac0207 (unittest.TestCase):

    def testDuration (self):
        dur = xsd.duration("P10675199DT2H48M5.4775807S")
        self.assertEqual(dur.days, 10675199)
        self.assertEqual(dur.seconds, 10085)
        self.assertEqual(dur.microseconds, 477580)
        serialized = pickle.dumps(dur)
        xdur = pickle.loads(serialized)
        self.assertEqual(dur, xdur)

    def testDateTime (self):
        now = xsd.dateTime.now()
        serialized = pickle.dumps(now)
        xnow = pickle.loads(serialized)
        self.assertEqual(now, xnow)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.utils.domutils

import resources

import unittest

class ExternalTrac0186 (unittest.TestCase):
    def testXBIngress (self):
        instance = resources.XBIngress(match='all', action1='none', digits1='', action2='none', digits2='')

    def testXBMatch (self):
        instance = resources.XBMatch('all')

if '__main__' == __name__:
    unittest.main()



from __future__ import print_function
xml_file = 'ipo.xml'

import pyxb.binding.saxer
import ipo

def ShowOrder (order):
    print('%s is sending %s %d thing(s):' % (order.billTo().name(), order.shipTo().name(), len(order.items().item())))
    for item in order.items().item():
        print('  Quantity %d of %s at $%s' % (item.quantity(), item.productName(), item.USPrice()))

if False:
    import pyxb.utils.domutils
    xmld = pyxb.utils.domutils.StringToDOM(open(xml_file).read())
    dom_value = ipo.CreateFromDOM(xmld.documentElement)
    ShowOrder(dom_value)

saxer = pyxb.binding.saxer.make_parser()
handler = saxer.getContentHandler()
saxer.parse(open(xml_file))
ShowOrder(handler.rootObject())



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xst = '''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="ival" type="xs:int"/>
  <xs:element name="sval" type="xs:string"/>
  <xs:complexType name="tInt">
    <xs:simpleContent>
      <xs:extension base="xs:int">
        <xs:attribute name="units" type="xs:string"/>
      </xs:extension>
    </xs:simpleContent>
  </xs:complexType>
  <xs:element name="Int" type="tInt"/>
  <xs:complexType name="tStr">
    <xs:simpleContent>
      <xs:extension base="xs:string">
        <xs:attribute name="version" type="xs:int"/>
      </xs:extension>
    </xs:simpleContent>
  </xs:complexType>
  <xs:element name="Str" type="tStr"/>
  <xs:complexType name="tMixed" mixed="true">
    <xs:sequence>
      <xs:element name="mString" type="xs:string"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="Mixed" type="tMixed"/>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0211 (unittest.TestCase):
    def testInvalidComplex (self):
        # Various values all being 4.
        bsv4 = ival(4)
        usv4 = ival.typeDefinition()(4)
        cv4 = Int(4)
        self.assertEqual(bsv4, 4)
        self.assertEqual(bsv4, usv4)
        self.assertEqual(bsv4, cv4.value())

        # Disallow creation from XML
        xmlt = six.u('<Int><ival>4</ival></Int>')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.NonElementValidationError, CreateFromDocument, xmlt)
        else:
            with self.assertRaises(pyxb.NonElementValidationError) as cm:
                instance = CreateFromDocument(xmlt)
            e = cm.exception
            self.assertTrue(isinstance(e.element, (ival.typeDefinition(), Node)))

        # Allow creation from unbound simple value
        instance = Int(usv4)
        self.assertEqual(instance.value(), 4)

        # Allow creation from bound simple value.  For now.
        instance = Int(bsv4)
        self.assertEqual(instance.value(), 4)

        # Disallow creation from complex value.
        self.assertRaises(pyxb.NonElementValidationError, Int, cv4)

    def testInvalidSimple (self):
        # Disallow creation from XML
        xmlt = six.u('<ival><ival>4</ival></ival>')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.NonElementValidationError, CreateFromDocument, xmlt)
        else:
            with self.assertRaises(pyxb.NonElementValidationError) as cm:
                instance = CreateFromDocument(xmlt)
            e = cm.exception
            self.assertTrue(isinstance(e.element, (ival.typeDefinition(), Node)))

        # Create a bound instance with value 4
        bv4 = ival(4)
        self.assertTrue(isinstance(bv4, ival.typeDefinition()))
        self.assertEqual(bv4._element(), ival)

        # Create an unbound instance with value 4
        uv4 = ival.typeDefinition()(4)
        self.assertTrue(isinstance(uv4, ival.typeDefinition()))
        self.assertTrue(uv4._element() is None)

        # OK to create from an unbound instance
        instance = ival(uv4)
        self.assertEqual(4, instance)

        # OK to create from a bound instance too.  For now.
        instance = ival(bv4)
        self.assertEqual(4, instance)

    def testBasicMixed (self):
        xmlt = six.u('<Mixed><mString>body</mString></Mixed>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        xmlt = six.u('<Mixed>pre<mString>body</mString>post</Mixed>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(3, len(instance.orderedContent()))
        nec = list(pyxb.NonElementContent(instance))
        self.assertEqual(2, len(nec))
        self.assertEqual(nec[0], six.u('pre'))
        self.assertEqual(nec[1], six.u('post'))

        # Yes, I know this is weird.  It's what PyXB does with this:
        # consume what's type-compatible as an element, and append the
        # rest as mixed content.
        instance = Mixed('body', 'post')
        self.assertEqual('body', instance.mString)
        self.assertEqual('post', ''.join(pyxb.NonElementContent(instance)))

        # This is more interesting: what isn't type-compatible gets to
        # be mixed content.
        instance = Mixed(4, 'body', 'post')
        self.assertEqual('body', instance.mString)
        self.assertEqual('4post', ''.join(pyxb.NonElementContent(instance)))

        # Even more interesting: a bound value is implicitly converted
        # to mixed content if the type doesn't match element content.
        bv4 = ival(4)
        instance = Mixed(bv4, 'body', 'post')
        self.assertEqual('body', instance.mString)
        self.assertEqual('4post', ''.join(pyxb.NonElementContent(instance)))
        oc = instance.orderedContent()
        self.assertEqual(3, len(oc))
        oc0 = oc[0].value
        self.assertTrue(isinstance(oc0, six.text_type))
        self.assertEqual(six.u('4'), oc0)

    def testBasicSimples (self):
        xmlt = six.u('<ival>4</ival>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance, 4)
        instance = ival(4)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance, 4)

        xmlt = six.u('<Int units="m">23</Int>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance.value(), 23)
        self.assertEqual(instance.units, six.u('m'))
        instance = Int(23, units="m")
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance.value(), 23)
        self.assertEqual(instance.units, six.u('m'))

        xmlt = six.u('<sval>text</sval>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance, six.u('text'))
        instance = sval(six.u('text'))
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance, six.u('text'))

        xmlt = six.u('<Str version="3">text</Str>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance.value(), six.u('text'))
        self.assertEqual(instance.version, 3)
        instance = Str(six.u('text'), version=3)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance.value(), six.u('text'))
        self.assertEqual(instance.version, 3)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd
from pyxb.utils import six
import binascii

class Test_hexBinary (unittest.TestCase):
    def testValues (self):
        data_values = [ '\x01', '\x00', '\x01\x23', '\x12\x34' ]
        for dt in data_values:
            dd = dt.encode('utf-8')
            v = xsd.hexBinary(dd)
            self.assertEqual(v, dd)

    def testStrings (self):
        encoded_values = [ six.u('01'), six.u('00'), six.u('ab'), six.u('Ab'), six.u('AB12') ]
        for et in encoded_values:
            ed = et.encode('utf-8')
            v = xsd.hexBinary.Factory(ed)
            self.assertEqual(v, ed)
            v = xsd.hexBinary.Factory(et, _from_xml=True)
            self.assertEqual(len(et)//2, len(v))
            self.assertEqual(et.upper(), v.xsdLiteral())
        v = xsd.hexBinary.Factory(six.u(''), _from_xml=True)
        self.assertEqual(0, len(v))
        v = xsd.hexBinary.Factory(six.u(' '), _from_xml=True)
        self.assertEqual(0, len(v))

    def testBadStrings (self):
        self.assertRaises(SimpleTypeValueError, xsd.hexBinary.Factory, six.u('0'), _from_xml=True)
        self.assertRaises(SimpleTypeValueError, xsd.hexBinary.Factory, six.u('012'), _from_xml=True)
        self.assertRaises(SimpleTypeValueError, xsd.hexBinary.Factory, six.u('01s'), _from_xml=True)
        self.assertRaises(SimpleTypeValueError, xsd.hexBinary.Factory, six.u('sb'), _from_xml=True)

    def testLiteralization (self):
        self.assertEqual('', xsd.hexBinary(''.encode('utf-8')).xsdLiteral())


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
import pyxb.binding.facets

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:simpleType name="tUC">
  <xs:annotation><xs:documentation>Simple type to represent an ASCII upper-case letter</xs:documentation></xs:annotation>
  <xs:restriction base="xs:string">
    <xs:pattern value="[A-Z]"/>
  </xs:restriction>
</xs:simpleType>
<xs:element name="UC" type="tUC"/>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0190 (unittest.TestCase):
    def testBasic (self):
        i = UC('A')
        self.assertEqual(i, 'A')
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.SimpleFacetValueError, UC, 'a')
        else:
            with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
                i = UC('a')
            e = cm.exception
            self.assertEqual(e.type, tUC)
            self.assertEqual(e.value, 'a')
            self.assertTrue(isinstance(e.facet, pyxb.binding.facets.CF_pattern))
            self.assertEqual(e.details(), 'Type tUC pattern constraint violated by value a')

    def testUnicode (self):
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(pyxb.SimpleFacetValueError, UC, six.unichr(0xf6))
        else:
            with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
                i = UC(six.unichr(0xf6))
            e = cm.exception
            self.assertEqual(e.type, tUC)
            self.assertEqual(e.value, six.unichr(0xf6))
            self.assertTrue(isinstance(e.facet, pyxb.binding.facets.CF_pattern))
            if six.PY2:
                self.assertRaises(UnicodeEncodeError, str, e.details())
            self.assertEqual(e.details(), six.u('Type tUC pattern constraint violated by value \xf6'))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="cards">
    <xs:restriction base="xs:string">
        <xs:enumeration value="clubs"/>
        <xs:enumeration value="hearts"/>
        <xs:enumeration value="diamonds"/>
        <xs:enumeration value="spades"/>
    </xs:restriction>
  </xs:simpleType>
  <xs:element name="card" type="cards"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0111 (unittest.TestCase):
    Expected = set( ('clubs', 'hearts', 'diamonds', 'spades') )
    def testItems (self):
        vals = set()
        for ee in six.iteritems(cards):
            self.assertTrue(isinstance(ee, cards._CF_enumeration._CollectionFacet_itemType))
            vals.add(ee.value())
        self.assertEqual(self.Expected, vals)

    def testIterItems (self):
        vals = set()
        for ee in six.iteritems(cards):
            self.assertTrue(isinstance(ee, cards._CF_enumeration._CollectionFacet_itemType))
            vals.add(ee.value())
        self.assertEqual(self.Expected, vals)

    def testValues (self):
        vals = set()
        for e in six.itervalues(cards):
            vals.add(e)
        self.assertEqual(self.Expected, vals)

    def testIterValues (self):
        vals = set()
        for e in six.itervalues(cards):
            vals.add(e)
        self.assertEqual(self.Expected, vals)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="intList">
    <xs:list itemType="xs:int"/>
  </xs:simpleType>
  <xs:element name="li" type="intList"/>
  <xs:complexType name="tAggregate">
    <xs:sequence>
      <xs:element ref="li"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="aggregate" type="tAggregate"/>
  <xs:complexType name="tMultiAggregate">
    <xs:sequence>
      <xs:element ref="li" maxOccurs="unbounded"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="multi" type="tMultiAggregate"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_200908041708 (unittest.TestCase):
    # This verifies that we can invoke xsdLiteral even when the value
    # stored in the content doesn't descend from _TypeBinding_mixin
    # (as long as it's compatible).
    def testSub (self):
        instance = aggregate(li=[])
        instance.li.append(1)
        self.assertTrue(instance.validateBinding())
        xmld = '<aggregate><li>1</li></aggregate>'.encode('utf-8')
        self.assertEqual(instance.toxml("utf-8", root_only=True), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.utils.domutils
from pyxb.utils import six

import bindings.s0 as s0
import bindings.s1 as s1

pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(s0.Namespace, 's0')
pyxb.utils.domutils.BindingDOMSupport.DeclareNamespace(s1.Namespace, 's1')

import unittest

class ExternalTrac0184 (unittest.TestCase):
    def testExtend0 (self):
        e0i = s0.e0i(32)
        e1i = s1.e1i(4)
        e1s = s1.e1s('ext0')
        e0extend0 = s0.e0extend0()
        self.assertRaises(pyxb.IncompleteElementContentError, e0extend0.toxml, 'utf-8')
        e0extend0.e0i = e0i
        xmlt = six.u('<s0:e0extend0 xmlns:s0="urn:s0add"><s0:e0i>32</s0:e0i></s0:e0extend0>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(e0extend0.toxml('utf-8', root_only=True), xmld)
        e0extend0.e1i = e1i
        xmlt = six.u('<s0:e0extend0 xmlns:s0="urn:s0add" xmlns:s1="urn:s1core"><s1:e1i>4</s1:e1i><s0:e0i>32</s0:e0i></s0:e0extend0>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(e0extend0.toxml('utf-8', root_only=True), xmld)
        e0extend0.e1s = e1s
        xmlt = six.u('<s0:e0extend0 xmlns:s0="urn:s0add" xmlns:s1="urn:s1core"><s1:e1i>4</s1:e1i><s1:e1s>ext0</s1:e1s><s0:e0i>32</s0:e0i></s0:e0extend0>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(e0extend0.toxml('utf-8', root_only=True), xmld)

    def testExtend1 (self):
        e0i = s0.e0i(32)
        e1i = s1.e1i(4)
        e1s = s1.e1s('ext1')
        e0extend1 = s0.e0extend1()
        self.assertRaises(pyxb.IncompleteElementContentError, e0extend1.toxml, 'utf-8')
        e0extend1.e0i = e0i
        xmlt = six.u('<s0:e0extend1 xmlns:s0="urn:s0add"><s0:e0i>32</s0:e0i></s0:e0extend1>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(e0extend1.toxml('utf-8', root_only=True), xmld)
        e0extend1.e1i = e1i
        xmlt = six.u('<s0:e0extend1 xmlns:s0="urn:s0add" xmlns:s1="urn:s1core"><s1:e1i>4</s1:e1i><s0:e0i>32</s0:e0i></s0:e0extend1>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(e0extend1.toxml('utf-8', root_only=True), xmld)
        e0extend1.e1s = e1s
        xmlt = six.u('<s0:e0extend1 xmlns:s0="urn:s0add" xmlns:s1="urn:s1core"><s1:e1s>ext1</s1:e1s><s1:e1i>4</s1:e1i><s0:e0i>32</s0:e0i></s0:e0extend1>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(e0extend1.toxml('utf-8', root_only=True), xmld)

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
import gc

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="name">
    <xs:restriction base="xs:string" />
  </xs:simpleType>
  <xs:element name="Container">
    <xs:complexType>
      <xs:attribute name="name" type="name"/>
      <xs:attribute name="name2" type="name"/>
    </xs:complexType>
  </xs:element>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestIssue0027 (unittest.TestCase):
    def testAll (self):
        st = name
        self.assertTrue(issubclass(st, xs.string))
        et = Container.typeDefinition()
        atu = et._AttributeMap.get('name')
        self.assertEqual(st, atu.dataType())
        atu = et._AttributeMap.get('name2')
        self.assertEqual(st, atu.dataType())

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import pyxb
import ipo
import xml.dom.minidom
import time
import io

xmld = open('ipo.xml', 'rb').read()

order = ipo.CreateFromDOM(xml.dom.minidom.parse(io.BytesIO(xmld)).documentElement)

print('%s is sending %s %d thing(s):' % (order.billTo().name(), order.shipTo().name(), len(order.items().item())))
for item in order.items().item():
    print('  Quantity %d of%s at $%s' % (item.quantity(), item.productName(), item.USPrice()))

# Give Mary more
try:
    item.setQuantity(100)
except pyxb.SimpleTypeValueError as e:
    print('Too many: %s' % (e,))
    item.setQuantity(10)
print('Increased quantity to %d' % (item.quantity(),))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six

from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/xsi-type.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

originalOneFloor = oneFloor
def oneFloorCtor (*args, **kw):
    return restaurant(*args, **kw)
originalOneFloor._SetAlternativeConstructor(oneFloorCtor)

from pyxb.exceptions_ import *

import unittest

class TestXSIType (unittest.TestCase):
    def testFailsNoType (self):
        xmlt = six.u('<elt/>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        self.assertRaises(pyxb.AbstractInstantiationError, CreateFromDOM, doc.documentElement)

    def testDirect (self):
        xmlt = six.u('<notAlt attrOne="low"><first>content</first></notAlt>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual('content', instance.first)
        self.assertEqual('low', instance.attrOne)

    def testSubstitutions (self):
        xmlt = six.u('<elt attrOne="low" xsi:type="alt1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><first>content</first></elt>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual('content', instance.first)
        self.assertEqual('low', instance.attrOne)
        xmlt = six.u('<elt attrTwo="hi" xsi:type="alt2" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><second/></elt>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertTrue(instance.second is not None)
        self.assertEqual('hi', instance.attrTwo)

    def testMultilevel (self):
        xmlt = six.u('<concreteBase><basement>dirt floor</basement></concreteBase>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual('dirt floor', instance.basement)
        xmlt = six.u('<oneFloor xsi:type="restaurant" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><basement>concrete</basement><lobby>tiled</lobby><room>eats</room></oneFloor>')
        doc = pyxb.utils.domutils.StringToDOM(xmlt)
        instance = CreateFromDOM(doc.documentElement)
        self.assertEqual(concreteBase_.basement, instance.__class__.basement)
        self.assertEqual(oneFloor_.lobby, instance.__class__.lobby)
        self.assertEqual(restaurant_.room, instance.__class__.room)
        self.assertEqual('tiled', instance.lobby)
        self.assertEqual('eats', instance.room)

    def testConstructor (self):
        kw = { 'basement' : 'concrete',
               'lobby' : 'tiled',
               'room' : 'eats' }
        ctd = restaurant_(**kw)
        dom = ctd.toDOM(element_name='restaurant').documentElement
        xmlt = six.u('<restaurant><basement>concrete</basement><lobby>tiled</lobby><room>eats</room></restaurant>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(dom.toxml("utf-8"), xmld)

        rest = restaurant(**kw)
        dom = rest.toDOM().documentElement
        self.assertEqual(dom.toxml("utf-8"), xmld)

        self.assertRaises(pyxb.AbstractInstantiationError, originalOneFloor, **kw)

    def testNesting (self):
        instance = block(oneFloor=[ restaurant(basement="dirt", lobby="tile", room="messy"),
                                    restaurant(basement="concrete", lobby="carpet", room="tidy")])
        self.assertEqual('dirt', instance.oneFloor[0].basement)
        self.assertEqual('messy', instance.oneFloor[0].room)
        self.assertEqual('concrete', instance.oneFloor[1].basement)
        self.assertEqual('tidy', instance.oneFloor[1].room)
        xmld = instance.toxml("utf-8")
        dom = pyxb.utils.domutils.StringToDOM(xmld)
        instance2 = CreateFromDOM(dom.documentElement)
        r2d = instance2.toxml("utf-8")
        r3d = instance2.toxml("utf-8")
        self.assertEqual(r2d, r3d)
        self.assertEqual(xmld, r2d)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node
import pyxb.binding.datatypes as xs

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:simpleType name="intList">
    <xs:list itemType="xs:int"/>
  </xs:simpleType>
  <xs:element name="li" type="intList"/>
  <xs:complexType name="tAggregate">
    <xs:sequence>
      <xs:element ref="li"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="aggregate" type="tAggregate"/>
  <xs:complexType name="tMultiAggregate">
    <xs:sequence>
      <xs:element ref="li" maxOccurs="unbounded"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="multi" type="tMultiAggregate"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

def SET_li (instance, v):
    instance.li = v

class TestTrac0032 (unittest.TestCase):
    """Storage of non-plural simple lists broken"""
    def testBasic (self):
        instance = aggregate()
        instance.li = [1,2,3]
        self.assertEqual(3, len(instance.li))
        self.assertEqual(1, len(instance.orderedContent()))
        self.assertTrue(instance.validateBinding())

        self.assertRaises(pyxb.SimpleTypeValueError, SET_li, instance, 1)
        self.assertRaises(pyxb.SimpleTypeValueError, SET_li, instance, [[1,2,3], [2,3,4]])

        instance = aggregate([1,2,3])
        self.assertEqual(3, len(instance.li))
        self.assertEqual(1, len(instance.orderedContent()))
        self.assertTrue(instance.validateBinding())

        instance = aggregate(li=[1,2,3])
        self.assertEqual(3, len(instance.li))
        self.assertEqual(1, len(instance.orderedContent()))
        self.assertTrue(instance.validateBinding())

        instance = aggregate()
        instance.li = []
        instance.li.append(1)
        instance.li.append(2)
        instance.li.append(3)
        self.assertEqual(3, len(instance.li))
        # Yes, the content is a single value; the members of the
        # simple type list are not visible at the content model level.
        self.assertEqual(1, len(instance.orderedContent()))
        self.assertTrue(instance.validateBinding())

    def testMulti (self):
        instance = multi()
        self.assertRaises(pyxb.SimpleTypeValueError, SET_li, instance, 1)
        self.assertRaises(pyxb.SimpleTypeValueError, SET_li, instance, [1, 2, 3])
        instance.li = [[1,2,3], [2,3,4]]
        self.assertEqual(2, len(instance.li))
        self.assertTrue(instance.validateBinding())
        self.assertEqual(1, len(instance.orderedContent()))

        instance = multi([1,2,3], [1,2,3]) # two li values
        self.assertEqual(2, len(instance.li))
        self.assertTrue(instance.validateBinding())
        self.assertEqual(2, len(instance.orderedContent()))

        # The individual lists here aren't recognized as li elements, so this becomes something unknown
        self.assertRaises(pyxb.MixedContentError, multi, [[1,2,3], [2,3,4]])

        self.assertRaises(pyxb.SimpleTypeValueError, multi, li=[1,2,3])

        instance = multi(li=[[1,2,3],[2,3,4]])
        self.assertEqual(2, len(instance.li))
        self.assertTrue(instance.validateBinding())
        self.assertEqual(1, len(instance.orderedContent()))

        instance = multi(li=[])
        instance.li.append(xs.int(1))
        self.assertRaises(pyxb.SimpleTypeValueError, instance.validateBinding)

        instance = CreateFromDocument('<multi><li>1</li></multi>')
        self.assertEqual(1, len(instance.li))
        self.assertTrue(instance.validateBinding())
        self.assertEqual(1, len(instance.orderedContent()))

        instance.li = []
        instance.li.append([1,2,3])
        instance.li.append([2,3,4])
        self.assertEqual(2, len(instance.li))
        self.assertTrue(instance.validateBinding())
        self.assertEqual(2, len(instance.orderedContent()))

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import sys
from pyxb.utils.six.moves.urllib import request as urllib_request
import GeoCoder
from pyxb import BIND
from pyxb.utils import domutils
import pyxb.bundles.wssplat.soap11 as soapenv
import pyxb.bundles.wssplat.soapenc as soapenc

address = '1600 Pennsylvania Ave., Washington, DC'
if 1 < len(sys.argv):
    address = sys.argv[1]

env = soapenv.Envelope(Body=BIND(GeoCoder.geocode(address)))

uri = urllib_request.Request('http://rpc.geocoder.us/service/soap/',
                      env.toxml("utf-8"),
                      { 'SOAPAction' : "http://rpc.geocoder.us/Geo/Coder/US#geocode", 'Content-Type': 'text/xml' } )

rxml = urllib_request.urlopen(uri).read()
#open('response.xml', 'w').write(rxml)
#rxml = open('response.xml').read()
response = soapenv.CreateFromDocument(rxml)

# OK, here we get into ugliness due to WSDL's concept of schema in the
# SOAP encoding not being consistent with XML Schema, even though it
# uses the same namespace.  See
# http://tech.groups.yahoo.com/group/soapbuilders/message/5879.  In
# short, the WSDL spec shows an example using soapenc:Array where a
# restriction was used to set the value of the wsdl:arrayType
# attribute.  This restriction failed to duplicate the element content
# of the base type, resulting in a content type of empty in the
# restricted type.  Consequently, PyXB can't get the information out
# of the DOM node, and we have to skip over the wildcard items to find
# something we can deal with.

# As further evidence the folks who designed SOAP 1.1 didn't know what
# they were doing, the encodingStyle attribute that's supposed to go
# in the Envelope can't validly be present there, since it's not
# listed and it's not in the namespace admitted by the attribute
# wildcard.  Fortunately, PyXB doesn't currently validate wildcards.

encoding_style = response.wildcardAttributeMap().get(soapenv.Namespace.createExpandedName('encodingStyle'))
items = []
if encoding_style == soapenc.Namespace.uri():
    gcr = response.Body.wildcardElements()[0]
    soap_array = gcr.wildcardElements()[0]
    items = soap_array.wildcardElements()
else:
    pass

for item in items:
    if (item.lat is None) or item.lat._isNil():
        print('Warning: Address did not resolve')
    print('''
%s %s %s %s %s
%s, %s  %s
%s %s''' % (item.number, item.prefix, item.street, item.type, item.suffix,
            item.city, item.state, item.zip,
            item.lat, item.long))
